readyCallbacks = [];
services.factory('DB', function($q) {
    var self = this;
    self.session = null;
    self.transaction;

    self.ready = function(callback){
        if(G_DB_ready){
            callback();
        }else{
            readyCallbacks.push(callback);
        }
    };

    self.DBready = function(){
        G_DB_ready = true;
        for(var i = 0; i < readyCallbacks.length; i++){
            readyCallbacks[i]();
        }
    };

    self.delete = function(callback, callbackerror) {
        window.sqlitePlugin.deleteDatabase("familia.db", callback, callbackerror);
    };

    self.initDB = function() {
        if(window.cordova){
            self.session = window.sqlitePlugin.openDatabase({name: "familia.db"});
            console.log('BANCO SQLITE');

            self.session.executeSql("PRAGMA foreign_keys = ON;", [],function (res) {
                console.log('CELK FOREIGN KEYS habilitadas: '+ JSON.stringify(res));
            });
        }else{
            self.session = window.openDatabase("familia.db", "1.0", "database", -1);
            console.log('BANCO WEBSQL');
        }
        console.log('CELK BANCO INICIADO');
    };

    self.runSqlQuery = function (query, errorMessage) {
        var q = $q.defer();
        self.query(query).then(function (){
            q.resolve();
        }, function(error){
            console.error(errorMessage + error.message);
            q.reject();
        });
        return q.promise;
    };

    self.executeSqlPromise = function(sql) {
        var q = $q.defer();

        self.transaction.executeSql(sql,[], function(transaction, result) {
            q.resolve();
        });

        return q.promise;
    };

    self.init = function(callback) {
        G_DB_ready = false;

        self.initDB();

        self.query("PRAGMA user_version;", []).then(function(result){
            var versaoAtual = parseInt(self.fetch(result).user_version);

            console.log('VERSAO ATUAL '+ versaoAtual);
            switch(versaoAtual + 1){
                case 1: migracaoInicial();
                case 2: migracao2();
                case 3: migracao3();
                case 4: migracao4();
                case 5: migracao5();
                case 6: migracao6();
                case 7: migracao7();
                case 8: migracao8();
                case 9: migracao9() //Foi gerado errado, por isso não tem nada.
                case 10: migracao10();
                case 11: migracao11();
                case 12: migracao12();
                case 13: migracao13();
                case 14: migracao14();
                case 15: migracao15();
                case 16: migracao16();
                case 17: migracao17();
                case 18: migracao18();
                case 19: migracao19();
                case 20: migracao20();

                versaoAtual = 20;//Deve receber o valor do ultimo case
            };

            self.createVersionTriggers();

            self.session.executeSql("PRAGMA user_version = '"+versaoAtual+"';", [],function (res) {
                //Chamar ready apos a ultima execucao
                console.log('TERMINOU DE EXECUTAR OS SQLS');

                self.DBready();
                if(callback){
                  callback();
                }
            });
        });
    };


    migracao20 = function(){
        console.log('MIGRACAO DE SQL 20');
        var sql = "UPDATE visitaDomiciliar SET desfecho = 3 WHERE EXISTS( SELECT 1 FROM domicilioEsus WHERE domicilioEsus.tipoImovel = 13 AND domicilioEsus.codigoDomicilio = visitaDomiciliar.codigoDomicilio ) AND visitaDomiciliar.codigoPaciente IS NULL AND visitaDomiciliar.desfecho = 1";
        self.query(sql).then(null, function(error){
            console.error('CELK Erro ao atualizar visitaDomiciliar: '+ error.message);
        });

    }

    migracao19 = function(){
        console.log('MIGRACAO DE SQL 19');
        var sql = "CREATE TABLE IF NOT EXISTS cep_brasil ( " +
        "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
        "codigoSistema INTEGER NOT NULL, " +
        "versao INTEGER NOT NULL, " +
        "cep VARCHAR NOT NULL)";
        self.query(sql).then(null, function(error){
            console.error('CELK Erro ao criar tabela cep_brasil: '+ error.message);
        });

        sql = "DELETE FROM visitaDomiciliar WHERE EXISTS (SELECT 1 FROM domicilioEsus WHERE tipoImovel not in (2,3,4,5,6,12,13) AND domicilioEsus.codigoDomicilio = visitaDomiciliar.codigoDomicilio) AND visitaDomiciliar.codigoPaciente is null";
        self.query(sql).then( null, function(error) {
            console.error('CELK Erro ao limpar dados da visitaDomicliar: ' + error.message);
        });


    }

    migracao18 = function() {

        console.log('MIGRACAO DE SQL 18');

        var sql = "DELETE FROM visitaDomiciliar WHERE EXISTS (SELECT 1 FROM domicilioEsus WHERE tipoImovel not in (2,3,4,5,6,12) AND domicilioEsus.codigoDomicilio = visitaDomiciliar.codigoDomicilio) AND visitaDomiciliar.codigoPaciente is null";

        self.query(sql).then( null, function(error) {
          console.error('CELK Erro ao limpar dados da visitaDomicliar: ' + error.message);
        });
      }

    migracao17 = function() {

      console.log('MIGRACAO DE SQL 17');

      var sql = "ALTER TABLE pesquisaVisita RENAME TO tmp_pesquisaVisita";

      self.query(sql).then(function() {

        sql = "CREATE TABLE IF NOT EXISTS pesquisaVisita ( " +
              "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
              "codigoResposta INTEGER NOT NULL, " +
              "codigoVisita INTEGER NOT NULL, " +
              "codigoPesquisaPergunta INTEGER NOT NULL, " +
              "codigoUsuario INTEGER NOT NULL, " +
              "FOREIGN KEY(codigoResposta) REFERENCES perguntaResposta(id), " +
              "FOREIGN KEY(codigoVisita) REFERENCES visitaDomiciliar(id), " +
              "FOREIGN KEY(codigoPesquisaPergunta) REFERENCES pesquisaPergunta(id), " +
              "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id))";

        self.query(sql).then(function() {

            sql = "INSERT INTO pesquisaVisita (codigoResposta, codigoVisita, codigoPesquisaPergunta, codigoUsuario) " +
                  "SELECT codigoResposta, codigoVisita, codigoPesquisaPergunta, codigoUsuario FROM tmp_pesquisaVisita";

            self.query(sql).then(function() {

                sql = "DROP TABLE tmp_pesquisaVisita";

                self.query(sql).then(null, function(error) {
                    console.error('CELK Erro ao deletar tmp_pesquisaVisita: ' + error.message);
                });
            }, function(error){
                console.error('CELK Erro ao inserir dados na tabela pesquisaVisita: ' + error.message);
            });
        }, function(error){
            console.error('CELK Erro ao recriar tabela pesquisaVisita: ' + error.message);
        });
      }, function(error) {
        console.error('CELK Erro ao renomear pesquisaVisita para tmp_pesquisaVisita: ' + error.message);
      });
    }

    migracao16 = function(){
        console.log('MIGRACAO DE SQL 16');
        //o SQLLite não possui uma solução para modificar as colunas então foi efetuado o seguinte

        //Renomeado a tabela original
        var sql = "ALTER TABLE visitaDomiciliar RENAME TO tmp_visitaDomiciliar";
        var errorMessage = "CELK Erro ao renomear visitaDomiciliar para tmp_visitaDomiciliar: ";

        self.runSqlQuery(sql, errorMessage).then(function() {
            //Criada outra com a estrutura identica exceto not null
            sql = "CREATE TABLE visitaDomiciliar"+
            " ( id INTEGER PRIMARY KEY AUTOINCREMENT, "+
            "codigoUsuario INTEGER NOT NULL,"+
            "codigoDomicilio INTEGER NOT NULL, "+
            "dataVisita INTEGER NOT NULL, "+
            "compartilhada INTEGER NOT NULL, "+
            "codigoPaciente INTEGER, "+
            "desfecho INTEGER NOT NULL, "+
            "motivoAtualizacao INTEGER, "+
            "motivoVisitaPeriodica INTEGER, "+
            "motivoBaConsulta INTEGER, "+
            "motivoBaExame INTEGER, "+
            "motivoBaVacina INTEGER, "+
            "motivoBaCondicionalidadesBolsaFamilia INTEGER, "+
            "motivoAcGestante INTEGER, "+
            "motivoAcPuerpera INTEGER, "+
            "motivoAcRecemNascido INTEGER, "+
            "motivoAcCrianca INTEGER, "+
            "motivoAcPessoaDesnutrida INTEGER, "+
            "motivoAcPessoaDeficiente INTEGER, "+
            "motivoAcPessoaHipertensa INTEGER, "+
            "motivoAcPessoaDiabetes INTEGER, "+
            "motivoAcPessoaAsmatica INTEGER, "+
            "motivoAcPessoaEnfisema INTEGER, "+
            "motivoAcPessoaCancer INTEGER, "+
            "motivoAcPessoaDoencaCronica INTEGER, "+
            "motivoAcPessoaHanseniase INTEGER, "+
            "motivoAcPessoaTuberculose INTEGER, "+
            "motivoAcAcamado INTEGER, "+
            "motivoAcVulnerabilidadeSocial INTEGER, "+
            "motivoAcCondicionalidadesBolsaFamilia INTEGER, "+
            "motivoAcSaudeMental INTEGER, "+
            "motivoAcUsuarioAlcool INTEGER, "+
            "motivoAcUsuarioDroga INTEGER, "+
            "motivoEgressoInternacao INTEGER, "+
            "motivoControleAmbientesVetores INTEGER, "+
            "motivoCampanhaSaude INTEGER, "+
            "motivoPrevencao INTEGER, "+
            "motivoOutros INTEGER, "+
            "dataColetaGps INTEGER, "+
            "horaColetaGps INTEGER, "+
            "latitude VARCHAR(50), "+
            "longitude VARCHAR(50), "+
            "versaoMobile INTEGER, "+
            "codigoSistema INTEGER, "+
            "peso DECIMAL(12, 2), "+
            "altura INTEGER, "+
            "uuidTablet VARCHAR, "+
            "motivoSintomaticosRespiratorios LONG, "+
            "motivoTabagista VARCHAR, "+
            "motivoAcaoEducativa INTEGER, "+
            "motivoImovelComFoco VARCHAR, "+
            "motivoAcaoMecanica VARCHAR, "+
            "motivoTratamentoFocal VARCHAR, "+
            "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id) "+
            "FOREIGN KEY(codigoDomicilio) REFERENCES domicilio(id) "+
            "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id));";
            errorMessage = "CELK Erro ao criar tabela visitaDomiciliar: ";

            self.runSqlQuery(sql, errorMessage).then(function() {
                //Inserir os dados da tabela renomeada
                sql = "INSERT INTO visitaDomiciliar (id, codigoUsuario, codigoDomicilio, dataVisita, compartilhada, codigoPaciente, desfecho, motivoAtualizacao, motivoVisitaPeriodica, motivoBaConsulta, motivoBaExame, motivoBaVacina, motivoBaCondicionalidadesBolsaFamilia, motivoAcGestante, motivoAcPuerpera, motivoAcRecemNascido, motivoAcCrianca, motivoAcPessoaDesnutrida, motivoAcPessoaDeficiente, motivoAcPessoaHipertensa, motivoAcPessoaDiabetes, motivoAcPessoaAsmatica, motivoAcPessoaEnfisema, motivoAcPessoaCancer, motivoAcPessoaDoencaCronica, motivoAcPessoaHanseniase, motivoAcPessoaTuberculose, motivoAcAcamado, motivoAcVulnerabilidadeSocial, motivoAcCondicionalidadesBolsaFamilia, motivoAcSaudeMental, motivoAcUsuarioAlcool, motivoAcUsuarioDroga, motivoEgressoInternacao, motivoControleAmbientesVetores, motivoCampanhaSaude, motivoPrevencao, motivoOutros, dataColetaGps, horaColetaGps, latitude, longitude, versaoMobile, codigoSistema, peso, altura, uuidTablet, motivoSintomaticosRespiratorios, motivoTabagista, motivoAcaoEducativa, motivoImovelComFoco, motivoAcaoMecanica, motivoTratamentoFocal)" +
                        "SELECT id, codigoUsuario, codigoDomicilio, dataVisita, compartilhada, codigoPaciente, desfecho, motivoAtualizacao, motivoVisitaPeriodica, motivoBaConsulta, motivoBaExame, motivoBaVacina, motivoBaCondicionalidadesBolsaFamilia, motivoAcGestante, motivoAcPuerpera, motivoAcRecemNascido, motivoAcCrianca, motivoAcPessoaDesnutrida, motivoAcPessoaDeficiente, motivoAcPessoaHipertensa, motivoAcPessoaDiabetes, motivoAcPessoaAsmatica, motivoAcPessoaEnfisema, motivoAcPessoaCancer, motivoAcPessoaDoencaCronica, motivoAcPessoaHanseniase, motivoAcPessoaTuberculose, motivoAcAcamado, motivoAcVulnerabilidadeSocial, motivoAcCondicionalidadesBolsaFamilia, motivoAcSaudeMental, motivoAcUsuarioAlcool, motivoAcUsuarioDroga, motivoEgressoInternacao, motivoControleAmbientesVetores, motivoCampanhaSaude, motivoPrevencao, motivoOutros, dataColetaGps, horaColetaGps, latitude, longitude, versaoMobile, codigoSistema, peso, altura, uuidTablet, motivoSintomaticosRespiratorios, motivoTabagista, motivoAcaoEducativa, motivoImovelComFoco, motivoAcaoMecanica, motivoTratamentoFocal"+
                        " FROM tmp_visitaDomiciliar";
                errorMessage = "CELK Erro ao inserir dados na tabela visitaDomiciliar: ";

                self.runSqlQuery (sql, errorMessage).then(function() {
                    //Apagar a tabela renomeada
                    sql = "DROP TABLE tmp_visitaDomiciliar";
                    errorMessage = "CELK Erro ao deletar tmp_visitaDomiciliar: ";
                    self.runSqlQuery(sql, errorMessage);
                });
            })
        });
    }

    migracao15 = function(){
        console.log('MIGRACAO DE SQL 15');

        var sql = "ALTER TABLE visitaDomiciliar ADD motivoSintomaticosRespiratorios LONG";
        var errorMessage = "CELK Erro ao adicionar motivoSintomaticosRespiratorios visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD motivoTabagista VARCHAR";
        errorMessage = "CELK Erro ao adicionar motivoTabagista visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD motivoAcaoEducativa INTEGER";
        errorMessage = "CELK Erro ao adicionar motivoAcaoEducativa visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD motivoImovelComFoco VARCHAR";
        errorMessage = "CELK Erro ao adicionar motivoImovelComFoco visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD motivoAcaoMecanica VARCHAR";
        errorMessage = "CELK Erro ao adicionar motivoAcaoMecanica visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD motivoTratamentoFocal VARCHAR";
        errorMessage = "CELK Erro ao adicionar motivoTratamentoFocal visitaDomiciliar: ";
    }

    migracao14 = function(){
        console.log('MIGRACAO DE SQL 14 #16452');

        // //DomicilioEsus
        // var sql = "ALTER TABLE domicilioEsus ADD flagForaArea LONG";
        sql = "y(sq";
            errorMessage = "error('CELK Erro ao adicionar flagForaArea domicilioEsus: ";
        // self.runSqlQuery(sql, errorMessage);

        var sql = "ALTER TABLE domicilioEsus ADD tipoImovel LONG";
        var errorMessage = "CELK Erro ao adicionar tipoImovel domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD nomeInstituicao VARCHAR";
        errorMessage = "CELK Erro ao adicionar nomeInstituicao domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD flagOutrosProfissionais INTEGER";
        errorMessage = "CELK Erro ao adicionar flagOutrosProfissionais domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD nomeResponsavelTecnico VARCHAR";
        errorMessage = "CELK Erro ao adicionar nomeResponsavelTecnico domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD numeroCartaoResponsavelTecnico VARCHAR";
        errorMessage = "CELK Erro ao adicionar numeroCartaoResponsavelTecnico domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD cargoResponsavelTecnico VARCHAR";
        errorMessage = "CELK Erro ao adicionar cargoResponsavelTecnico domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD telefoneResponsavelTecnico VARCHAR";
        errorMessage = "CELK Erro ao adicionar telefoneResponsavelTecnico domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracao13 = function(){
        console.log('MIGRACAO DE SQL 13 #16447');

        //PacienteEsus
        var sql = "ALTER TABLE pacienteEsus ADD numeroDO VARCHAR";
        var errorMessage = "CELK Erro ao adicionar numeroDO pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD dataObito VARCHAR";
        errorMessage = "CELK Erro ao adicionar dataObito pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD identidadeGenero VARCHAR";
        errorMessage = "CELK Erro ao adicionar identidadeGenero pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracao12 = function(){
        console.log('MIGRACAO DE SQL 12 #16447');

        //PacienteEsus
        var sql = "ALTER TABLE pacienteEsus ADD paiDesconhecido INTEGER";
        var errorMessage = "CELK Erro ao adicionar paiDesconhecido pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD maeDesconhecida INTEGER";
        errorMessage = "CELK Erro ao adicionar maeDesconhecida pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD dataNaturalizacao VARCHAR";
        errorMessage = "CELK Erro ao adicionar dataNaturalizacao pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD portariaNaturalizacao VARCHAR";
        errorMessage = "CELK Erro ao adicionar portariaNaturalizacao pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD dtEntradaBrasil VARCHAR";
        errorMessage = "CELK Erro ao adicionar dtEntradaBrasil pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        // var sql = "ALTER TABLE pacienteEsus ADD flagForaArea INTEGER";
        sql = "y(sq";
            errorMessage = "error('CELK Erro ao adicionar flagForaArea pacienteEsus: ";
        // self.runSqlQuery(sql, errorMessage);

        //Paciente
        sql = "ALTER TABLE paciente ADD nomePai VARCHAR";
        errorMessage = "CELK Erro ao adicionar nomePai pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracao11 = function(){
            console.log('MIGRACAO DE SQL 11');

            var sql = "ALTER TABLE domicilio ADD visitaNoMes integer";
            var errorMessage = "CELK Erro ao adicionar visitaNoMes em domicilio: ";
            self.runSqlQuery(sql, errorMessage);
        };

    migracao10 = function(){
        console.log('MIGRACAO DE SQL 10');

        var sql = "ALTER TABLE endereco ADD uuidTablet VARCHAR";
        var errorMessage = "CELK Erro ao adicionar uuid em endereco: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilio ADD uuidTablet VARCHAR";
        errorMessage = "CELK Erro ao adicionar uuid em domicilio: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE domicilioEsus ADD uuidTablet VARCHAR";
        errorMessage = "CELK Erro ao adicionar uuid em domicilio esus: ";
        self.runSqlQuery(sql, errorMessage);

        var sql = "ALTER TABLE paciente ADD uuidTablet VARCHAR";
        errorMessage = "CELK Erro ao adicionar uuid em paciente: ";
        self.runSqlQuery(sql, errorMessage);

        var sql = "ALTER TABLE visitaDomiciliar ADD uuidTablet VARCHAR";
        errorMessage = "CELK Erro ao adicionar uuid em visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);
    };

    migracao9 = function(){
        //Continuar daqui
    }

    migracao8 = function(){
        console.log('MIGRACAO DE SQL 8');

        var sql = "CREATE TABLE IF NOT EXISTS pesquisa ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR NOT NULL, "+
                "inicio INTEGER NOT NULL, "+
                "fim INTEGER NOT NULL, "+
                "status INTEGER NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela pesquisa: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_pesquisa ON pesquisa(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_pesquisa: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS perguntaPesquisa ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR NOT NULL)";
        errorMessage = "CELK Erro ao criar tabela perguntaPesquisa: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_perguntaPesquisa ON perguntaPesquisa(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_perguntaPesquisa: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS pesquisaPergunta ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "codigoPesquisa INTEGER NOT NULL, "+
                "codigoPergunta INTEGER NOT NULL, "+
                "FOREIGN KEY(codigoPesquisa) REFERENCES pesquisa(id) "+
                "FOREIGN KEY(codigoPergunta) REFERENCES perguntaPesquisa(id))";
        errorMessage = "CELK Erro ao criar tabela pesquisaPergunta: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_pesquisaPergunta ON pesquisaPergunta(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_pesquisaPergunta: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS perguntaResposta ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR NOT NULL, "+
                "codigoPergunta INTEGER NOT NULL, "+
                "FOREIGN KEY(codigoPergunta) REFERENCES perguntaPesquisa(id))";
        errorMessage = "CELK Erro ao criar tabela perguntaResposta: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_perguntaResposta ON perguntaResposta(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_perguntaResposta: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS pesquisaVisita ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoResposta INTEGER NOT NULL, " +
                "codigoVisita INTEGER NOT NULL, " +
                "codigoPesquisaPergunta INTEGER NOT NULL, " +
                "codigoUsuario INTEGER NOT NULL, " +
                "FOREIGN KEY(codigoResposta) REFERENCES perguntaResposta(id),"+
                "FOREIGN KEY(codigoVisita) REFERENCES visitaDomiciliar(id),"+
                "FOREIGN KEY(codigoPesquisaPergunta) REFERENCES pesquisaPergunta(id),"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id))";
        errorMessage = "CELK Erro ao criar tabela pesquisaVisita: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD codigoSistema INTEGER;";
        errorMessage = "CELK Erro ao adicionar codigoSistema: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD peso DECIMAL(12, 2);";
        errorMessage = "CELK Erro ao adicionar peso: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE visitaDomiciliar ADD altura INTEGER;";
        errorMessage = "CELK Erro ao adicionar altura: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS dominioPaciente ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER, " +
                "versao INTEGER , " +
                "nome VARCHAR , "+
                "dataNascimento VARCHAR , "+
                "nomeMae VARCHAR, "+
                "cns VARCHAR, "+
                "cpf VARCHAR, "+
                "descricaoFamilia VARCHAR,"+
                "sexo VARCHAR,"+
                "ativo VARCHAR,"+
                "keyword VARCHAR,"+
                "codigoInterno INTEGER,"+
                "codigoMicroArea INTEGER,"+
                "codigoArea INTEGER)";
        errorMessage = "CELK Erro ao criar tabela dominioPaciente: ";
        self.runSqlQuery(sql, errorMessage);
    };

    migracao7 = function(){
        console.log('MIGRACAO DE SQL 7');

        var sql = "ALTER TABLE paciente ADD registroVacina INTEGER;";
        var errorMessage = "CELK Erro ao adicionar registroVacina na tabela paciente: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS vacinasImagens ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoPaciente INTEGER NOT NULL, " +
                "codigoUsuario INTEGER NOT NULL, " +
                "imagem VARCHAR NOT NULL, "+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id)"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id))";
        errorMessage = "CELK Erro ao criar tabela vacinasImagens: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_vac_imagens ON vacinasImagens(codigoPaciente)";
        errorMessage = "CELK Erro ao criar indice idx_vac_imagens: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS tipoVacina ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR NOT NULL)";
        errorMessage = "CELK Erro ao criar tabela tipoVacina: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_tipo_vacina ON tipoVacina(descricao)";
        errorMessage = "CELK Erro ao criar indice idx_tipo_vacina: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS registroVacinas ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoPaciente INTEGER NOT NULL, " +
                "codigoTipoVacina INTEGER NOT NULL, "+
                "codigoUsuario INTEGER NOT NULL, "+
                "dataAplicacao INTEGER NOT NULL, "+
                "lote VARCHAR, "+
                "observacao VARCHAR, "+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id), "+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id), "+
                "FOREIGN KEY(codigoTipoVacina) REFERENCES tipoVacina(id))";
        errorMessage = "CELK Erro ao criar tabela registroVacinas: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_reg_vac ON registroVacinas(codigoPaciente)";
        errorMessage = "CELK Erro ao criar indice idx_reg_vac: ";
        self.runSqlQuery(sql, errorMessage);
    };

    migracao6 = function(){
        console.log('MIGRACAO DE SQL 6');

        var sql = "CREATE TABLE IF NOT EXISTS motivoNaoComparecimento ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "outros INTEGER NOT NULL, " +
                "descricao VARCHAR(50) NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela motivoNaoComparecimento: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE TABLE IF NOT EXISTS notificacaoAgendas ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "codigoPaciente INTEGER NOT NULL, " +
                "codigoAgenda INTEGER NOT NULL, " +
                "tipoAgenda VARCHAR, " +
                "local VARCHAR, " +
                "dataAgendamento INTEGER, " +
                "status INTEGER NULL,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "statusMobile INTEGER NOT NULL DEFAULT 0, "+//0-ABERTO 1-COMPARECEU 2-NAO_COMPARECEU
                "codigoMotivo INTEGER, "+
                "descricaoOutroMotivo VARCHAR(50), "+
                "codigoUsuario INTEGER, "+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id),"+
                "FOREIGN KEY(codigoMotivo) REFERENCES motivoNaoComparecimento(id))";
        errorMessage = "CELK Erro ao criar tabela notificacaoAgendas: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_not_agendas_1 ON notificacaoAgendas(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_not_agendas_1: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_not_agendas_2 ON notificacaoAgendas(codigoAgenda)";
        errorMessage = "CELK Erro ao criar indice idx_not_agendas_2: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_not_agendas_3 ON notificacaoAgendas(versaoMobile)";
        errorMessage = "CELK Erro ao criar indice idx_not_agendas_3: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracao5 = function(){
        console.log('MIGRACAO DE SQL 5');

        var sql = "CREATE TABLE IF NOT EXISTS notificacaoPaciente ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "codigoPaciente INTEGER NOT NULL, " +
                "vacinaAtrasada INTEGER NULL)";
        var errorMessage = "CELK Erro ao criar tabela notificacaoPaciente: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_not_paciente_1 ON notificacaoPaciente(codigoSistema)"
        errorMessage = "CELK Erro ao criar indice idx_not_paciente_1: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_not_paciente_2 ON notificacaoPaciente(codigoPaciente)"
        errorMessage = "CELK Erro ao criar indice idx_not_paciente_2: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracao4 = function(){
        console.log('MIGRACAO DE SQL 4');

        var sql = "CREATE TABLE IF NOT EXISTS etniaIndigena ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "descricao VARCHAR)";
        var errorMessage = "CELK Erro ao criar tabela etniaIndigena: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_etnia_indigena ON etniaIndigena(codigoSistema)"
        errorMessage = "CELK Erro ao criar indice idx_etnia_indigena: "
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE paciente ADD COLUMN codigoEtniaIndigena INTEGER;"
        errorMessage = "CELK Erro ao criar coluna codigoEtniaIndigena: "
        self.runSqlQuery(sql, errorMessage);
    };

    migracao3 = function(){
        console.log('MIGRACAO DE SQL 3');

        var sql = "CREATE TABLE IF NOT EXISTS pacienteDado ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "versao INTEGER NULL, " +
                "dataColeta INTEGER NOT NULL, " +
                "peso INTEGER,"+
                "altura INTEGER,"+
                "codigoPaciente INTEGER NOT NULL,"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id))";
        var errorMessage = "CELK Erro ao criar tabela pacienteDado: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_paciente_dado ON pacienteDado(codigoPaciente)";
        errorMessage = "CELK Erro ao criar indice idx_paciente_dado: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracao2 = function(){
        console.log('MIGRACAO DE SQL 2');

        sql = "ALTER TABLE paciente ADD COLUMN prontuario varchar(30);";
        errorMessage = "CELK Erro ao criar coluna prontuario: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE paciente ADD COLUMN motivoExclusaoDomicilio INTEGER;";
        errorMessage = "CELK Erro ao criar coluna motivoExclusaoDomicilio: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE paciente ADD COLUMN motivoExclusao INTEGER;";
        errorMessage = "CELK Erro ao criar coluna motivoExclusao: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "ALTER TABLE pacienteEsus ADD COLUMN parentescoResponsavel INTEGER;";
        errorMessage = "CELK Erro ao criar coluna parentescoResponsavel: ";
        self.runSqlQuery(sql, errorMessage);

    };

    migracaoInicial = function(){
        self.tabelaCbo();
        self.tabelaProfissional();
        self.tabelaUsuarios();
        self.tabelaOrgaoEmissor();
        self.tabelaRaca();
        self.tabelaTipoLogradouro();
        self.tabelaPais();
        self.tabelaEstado();
        self.tabelaCidade();
        self.tabelaEquipe();
        self.tabelaMicroArea();
        self.tabelaEquipeProfissional();
        self.tabelaEndereco();
        self.tabelaDomicilio();
        self.tabelaDomicilioEsus();
        self.tabelaPaciente();
        self.tabelaPacienteResponsavelAux();
        self.tabelaCns();
        self.tabelaDocumentos();
        self.tabelaPacienteEsus();
        self.tabelaVisitaDomiciliar();
        self.tabelaErroDomicilio();
        self.tabelaErroPaciente();
    };

    self.deleteVersionTriggers = function(){
        var deferred = $q.defer();

        self.session.transaction(function(tx) {
            self.transaction = tx;
            self.executeSqlPromise("DROP TRIGGER IF EXISTS v_paciente_U;")
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_paciente_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_pacienteEsus_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_pacienteEsus_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_documentos_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_documentos_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_cns_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_cns_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_domicilio_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_domicilio_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_domicilioEsus_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_domicilioEsus_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_endereco_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_endereco_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_pacienteDado_U;"))
            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_pacienteDado_I;"))

            .then(self.executeSqlPromise ("DROP TRIGGER IF EXISTS v_notificacaoAgendas_U;"))
            .then(function() {
                console.log('CELK Triggers deletadas');
                return deferred.resolve();
            });

        });
        return deferred.promise;
    };

    self.createVersionTriggers = function(){
        var deferred = $q.defer();

        self.session.transaction(function(tx) {
            self.transaction = tx;
            self.executeSqlPromise("CREATE TRIGGER IF NOT EXISTS v_paciente_U AFTER UPDATE ON paciente BEGIN UPDATE paciente SET versaoMobile = (SELECT MAX(versaoMobile) FROM paciente)+1 WHERE id = NEW.id; END;")
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_paciente_I AFTER INSERT ON paciente BEGIN UPDATE paciente SET versaoMobile = (SELECT MAX(versaoMobile) FROM paciente)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_pacienteEsus_U AFTER UPDATE ON pacienteEsus BEGIN UPDATE pacienteEsus SET versaoMobile = (SELECT MAX(versaoMobile) FROM pacienteEsus)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_pacienteEsus_I AFTER INSERT ON pacienteEsus BEGIN UPDATE pacienteEsus SET versaoMobile = (SELECT MAX(versaoMobile) FROM pacienteEsus)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_documentos_U AFTER UPDATE ON documentos BEGIN UPDATE documentos SET versaoMobile = (SELECT MAX(versaoMobile) FROM documentos)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_documentos_I AFTER INSERT ON documentos BEGIN UPDATE documentos SET versaoMobile = (SELECT MAX(versaoMobile) FROM documentos)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_cns_U AFTER UPDATE ON cns BEGIN UPDATE cns SET versaoMobile = (SELECT MAX(versaoMobile) FROM cns)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_cns_I AFTER INSERT ON cns BEGIN UPDATE cns SET versaoMobile = (SELECT MAX(versaoMobile) FROM cns)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_domicilio_U AFTER UPDATE ON domicilio BEGIN UPDATE domicilio SET versaoMobile = (SELECT MAX(versaoMobile) FROM domicilio)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_domicilio_I AFTER INSERT ON domicilio BEGIN UPDATE domicilio SET versaoMobile = (SELECT MAX(versaoMobile) FROM domicilio)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_domicilioEsus_U AFTER UPDATE ON domicilioEsus BEGIN UPDATE domicilioEsus SET versaoMobile = (SELECT MAX(versaoMobile) FROM domicilioEsus)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_domicilioEsus_I AFTER INSERT ON domicilioEsus BEGIN UPDATE domicilioEsus SET versaoMobile = (SELECT MAX(versaoMobile) FROM domicilioEsus)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_endereco_U AFTER UPDATE ON endereco BEGIN UPDATE endereco SET versaoMobile = (SELECT MAX(versaoMobile) FROM endereco)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_endereco_I AFTER INSERT ON endereco BEGIN UPDATE endereco SET versaoMobile = (SELECT MAX(versaoMobile) FROM endereco)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_pacienteDado_U AFTER UPDATE ON pacienteDado BEGIN UPDATE pacienteDado SET versaoMobile = (SELECT MAX(versaoMobile) FROM pacienteDado)+1 WHERE id = NEW.id; END;"))
            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_pacienteDado_I AFTER INSERT ON pacienteDado BEGIN UPDATE pacienteDado SET versaoMobile = (SELECT MAX(versaoMobile) FROM pacienteDado)+1 WHERE id = NEW.id; END;"))

            .then(self.executeSqlPromise ("CREATE TRIGGER IF NOT EXISTS v_notificacaoAgendas_U AFTER UPDATE ON notificacaoAgendas BEGIN UPDATE notificacaoAgendas SET versaoMobile = NEW.versaoMobile +1 WHERE id = NEW.id; END;"))
            .then(function() {
                console.log('CELK Triggers criada');
                    return deferred.resolve();
            });
        });

        return deferred.promise;
    };

    self.tabelaProfissional = function(){
        var sql = "CREATE TABLE IF NOT EXISTS profissional ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "nome VARCHAR(60) NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela profissional: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_profissional ON profissional(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_profissional: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_profissional ON profissional(codigoSistema)";
        errorMessage = "CREATE UNIQUE INDEX IF NOT EXISTS idx_profissional ON profissional(codigoSistema)";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaCbo = function(){
        var sql = "CREATE TABLE IF NOT EXISTS cbo ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema VARCHAR(10) NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR(150) NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela cbo: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_cbo ON cbo(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_cbo: ";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaOrgaoEmissor = function(){
        var sql = "CREATE TABLE IF NOT EXISTS orgaoEmissor ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR(60) NOT NULL, "+
                "sigla VARCHAR(6))";
        var errorMessage = "CELK Erro ao criar tabela orgaoEmissor: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_orgaoEmissor ON orgaoEmissor(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_orgaoEmissor: ";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaRaca = function(){
        var sql = "CREATE TABLE IF NOT EXISTS raca ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR(30) NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela raca: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_raca ON raca(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_raca: ";
        self.runSqlQuery(sql, errorMessage)

    };

    self.tabelaTipoLogradouro = function(){
        var sql = "CREATE TABLE IF NOT EXISTS tipoLogradouro ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR(72) NOT NULL, "+
                "ordem INTEGER NOT NULL DEFAULT 999, "+
                "sigla VARCHAR(15))";
        var errorMessage = "CELK Erro ao criar tabela tipoLogradouro: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_tipoLogradouro ON tipoLogradouro(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_tipoLogradouro: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_tipoLogradouro_2 ON tipoLogradouro(ordem,descricao)";
        errorMessage = "CELK Erro ao criar indice idx_tipoLogradouro_2: ";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaPais = function(){
        var sql = "CREATE TABLE IF NOT EXISTS pais ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "ordem INTEGER NOT NULL DEFAULT 999, " +
                "descricao VARCHAR(50) NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela pais: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_pais ON pais(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_pais: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_pais_2 ON pais(descricao)";
        errorMessage = "CELK Erro ao criar indice idx_pais_2:";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_pais_3 ON pais(ordem, descricao)";
        errorMessage = "CELK Erro ao criar indice idx_pais_3: ";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaEstado = function(){
        var sql = "CREATE TABLE IF NOT EXISTS estado ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR(40) NOT NULL, " +
                "sigla VARCHAR(2) NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela estado: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_estado ON estado(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_estado: ";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaCidade = function(){
        var sql = "CREATE TABLE IF NOT EXISTS cidade ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "descricao VARCHAR(50) NOT NULL, " +
                "codigoEstado INTEGER NOT NULL, " +
                "FOREIGN KEY(codigoEstado) REFERENCES estado(id))";
        var errorMessage = "CELK Erro ao criar tabela cidade: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_cidade ON cidade(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_cidade: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_cidade_2 ON cidade(descricao)";
        errorMessage = "CELK Erro ao criar indice idx_cidade_2: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_cidade_3 ON cidade(codigoEstado)";
        errorMessage = "CELK Erro ao criar indice idx_cidade_3: ";
        self.runSqlQuery(sql, errorMessage);

    };

    self.tabelaUsuarios = function(){
        var sql = "CREATE TABLE IF NOT EXISTS usuarios ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "nome VARCHAR(50) NOT NULL, " +
                "login VARCHAR(20) NOT NULL, " +
                "senha VARCHAR(200) NOT NULL, " +
                "status VARCHAR(1) NOT NULL, " +
                "codigoProfissional INTEGER, " +
                "nivel VARCHAR(1), "+
                "FOREIGN KEY(codigoProfissional) REFERENCES profissional(id))";
        var errorMessage = "CELK Erro ao criar tabela usuarios: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_usuarios ON usuarios(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_usuarios: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaEquipe = function(){
        var sql = "CREATE TABLE IF NOT EXISTS equipe ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "codigoEmpresa INTEGER, " +
                "descricao VARCHAR(60))";
        var errorMessage = "CELK Erro ao criar tabela usuarios: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_equipe ON equipe(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_equipe: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaMicroArea = function(){
        var sql = "CREATE TABLE IF NOT EXISTS microArea ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "microArea INTEGER)";
        var errorMessage = "CELK Erro ao criar tabela microArea: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_microArea ON microArea(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_microArea: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaEquipeProfissional = function(){
        var sql = "CREATE TABLE IF NOT EXISTS equipeProfissional ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NOT NULL, " +
                "versao INTEGER NOT NULL, " +
                "codigoProfissional INTEGER,"+
                "codigoMicroArea INTEGER,"+
                "codigoEquipe INTEGER,"+
                "status INTEGER,"+
                "FOREIGN KEY(codigoProfissional) REFERENCES profissional(id),"+
                "FOREIGN KEY(codigoMicroArea) REFERENCES microArea(id),"+
                "FOREIGN KEY(codigoEquipe) REFERENCES equipe(id))";
        var errorMessage = "CELK Erro ao criar tabela equipeProfissional: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_equipeProfissional ON equipeProfissional(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_equipeProfissional: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaEndereco = function(){
        var sql = "CREATE TABLE IF NOT EXISTS endereco ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "codigoCidade INTEGER,"+
                "cep VARCHAR(10),"+
                "bairro VARCHAR(30),"+
                "codigoTipoLogradouro INTEGER,"+
                "logradouro VARCHAR(50),"+
                "complementoLogradouro VARCHAR(50),"+
                "numeroLogradouro VARCHAR(7),"+
                "telefone VARCHAR(15),"+
                "telefoneReferencia VARCHAR(15),"+
                "pontoReferencia VARCHAR(60),"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoCidade) REFERENCES cidade(id),"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoTipoLogradouro) REFERENCES tipoLogradouro(id))";
        var errorMessage = "CELK Erro ao criar tabela endereco: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_endereco ON endereco(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_endereco: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_endereco_2 ON endereco(logradouro)";
        errorMessage = "CELK Erro ao criar indice idx_endereco_2: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaDomicilio = function(){
        var sql = "CREATE TABLE IF NOT EXISTS domicilio ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "codigoEndereco INTEGER,"+
                "numeroFamilia INTEGER,"+
                "codigoMicroArea INTEGER,"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoEndereco) REFERENCES endereco(id),"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoMicroArea) REFERENCES microArea(id))";
        var errorMessage = "CELK Erro ao criar tabela domicilio: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_domicilio ON domicilio(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_domicilio: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaDomicilioEsus = function(){
        var sql = "CREATE TABLE IF NOT EXISTS domicilioEsus ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "codigoDomicilio INTEGER NOT NULL,"+
                "situacaoMoradia INTEGER,"+
                "localizacao INTEGER,"+
                "tipoDomicilio INTEGER,"+
                "numeroMoradores INTEGER,"+
                "numeroComodos INTEGER,"+
                "condicaoUsoTerra INTEGER,"+
                "tipoAcessoDomicilio INTEGER,"+
                "materialDominante INTEGER,"+
                "possuiEnergiaEletrica INTEGER,"+
                "abastecimentoAgua INTEGER,"+
                "tratamentoAgua INTEGER,"+
                "esgotamento INTEGER,"+
                "destinoLixo INTEGER,"+
                "gato INTEGER,"+
                "cachorro INTEGER,"+
                "passaro INTEGER,"+
                "criacao INTEGER,"+
                "outros INTEGER,"+
                "quantos INTEGER,"+
                "dataColetaGps INTEGER,"+
                "latitude VARCHAR(50),"+
                "longitude VARCHAR(50),"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoDomicilio) REFERENCES domicilio(id))";
        var errorMessage = "CELK Erro ao criar tabela domicilioEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_domicilioesus ON domicilioEsus(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_domicilioesus: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaPaciente = function(){
        var sql = "CREATE TABLE IF NOT EXISTS paciente ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "excluido VARCHAR(1) NOT NULL,"+
                "nome VARCHAR(70) NOT NULL,"+
                "dataNascimento INTEGER NOT NULL,"+
                "sexo VARCHAR(1) NOT NULL,"+
                "nomeMae VARCHAR(70),"+
                "codigoCidade INTEGER,"+
                "cpf VARCHAR(14),"+
                "rg VARCHAR(20),"+
                "telefone VARCHAR(15),"+
                "celular VARCHAR(15),"+
                "codigoRaca INTEGER,"+
                "codigoPais INTEGER,"+
                "email VARCHAR(100),"+
                "apelido VARCHAR(50),"+
                "flagResponsavelFamiliar INTEGER,"+
                "codigoResponsavelFamiliar INTEGER,"+
                "nacionalidade INTEGER,"+
                "codigoDomicilio INTEGER,"+
                "telefone2 VARCHAR(15),"+
                "telefone3 VARCHAR(15),"+
                "telefone4 VARCHAR(15),"+
                "religiao VARCHAR(50),"+
                "localTrabalho VARCHAR(50),"+
                "telefoneTrabalho VARCHAR(15),"+
                "responsavel VARCHAR(70),"+
                "parentescoResponsavel VARCHAR(20),"+
                "urgenciaNome VARCHAR(70),"+
                "urgenciaTelefone VARCHAR(15),"+
                "urgenciaParentesco VARCHAR(20),"+
                "rendaFamiliar INTEGER,"+
                "resideDesde INTEGER,"+
                "situacao INTEGER,"+
                "nis VARCHAR,"+
                "keyword VARCHAR(200),"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoCidade) REFERENCES cidade(id),"+
                "FOREIGN KEY(codigoRaca) REFERENCES raca(id),"+
                "FOREIGN KEY(codigoPais) REFERENCES pais(id),"+
                "FOREIGN KEY(codigoResponsavelFamiliar) REFERENCES paciente(id),"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoDomicilio) REFERENCES domicilio(id))";
        var errorMessage = "CELK Erro ao criar tabela paciente: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_paciente ON paciente(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_paciente: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_paciente_cs ON paciente(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_paciente_cs: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_paciente_nm ON paciente(nome)";
        errorMessage = "CELK Erro ao criar indice idx_paciente_nm: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_paciente_keyword ON paciente(keyword)";
        errorMessage = "CELK Erro ao criar indice idx_paciente_keyword: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_paciente_dom ON paciente(codigoDomicilio)";
        errorMessage = "CELK Erro ao criar indice idx_paciente_dom: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE INDEX IF NOT EXISTS idx_paciente_versao ON paciente(versaoMobile)";
        errorMessage = "CELK Erro ao criar indice idx_paciente_versao: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaPacienteResponsavelAux = function(){
        var sql = "CREATE TABLE IF NOT EXISTS pacienteResponsavelAux ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoPacienteSistema INTEGER NOT NULL, " +
                "codigoResponsavelSistema INTEGER NOT NULL)";
        var errorMessage = "CELK Erro ao criar tabela pacienteResponsavelAux: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_pacienteResponsavelAux ON pacienteResponsavelAux(codigoPacienteSistema)";
        errorMessage = "CELK Erro ao criar indice idx_pacienteResponsavelAux: ";
        self.runSqlQuery(sql, errorMessage);
};

    self.tabelaCns = function(){
        var sql = "CREATE TABLE IF NOT EXISTS cns ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "excluido VARCHAR(1) NOT NULL,"+
                "numeroCartao VARCHAR(70) NOT NULL,"+
                "codigoPaciente INTEGER NOT NULL,"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id))";
        var errorMessage = "CELK Erro ao criar tabela CNS: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_cns ON cns(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_cns: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_cns_2 ON cns(versaoMobile)";
        errorMessage = "CELK Erro ao criar indice idx_cns_2: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaDocumentos = function(){
        var sql = "CREATE TABLE IF NOT EXISTS documentos ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "excluido VARCHAR(1) NOT NULL,"+
                "tipoDocumento INTEGER NOT NULL,"+
                "numeroDocumento VARCHAR(13),"+
                "complemento VARCHAR(4),"+
                "codigoOrgaoEmissor INTEGER,"+
                "numeroCartorio VARCHAR(20),"+
                "numeroLivro VARCHAR(8),"+
                "numeroFolha VARCHAR(4),"+
                "numeroTermo VARCHAR(8),"+
                "numeroMatricula VARCHAR(50),"+
                "dataEmissao INTEGER,"+
                "siglaUf VARCHAR(2),"+
                "codigoPaciente INTEGER NOT NULL,"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoOrgaoEmissor) REFERENCES orgaoEmissor(id),"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id))";
        var errorMessage = "CELK Erro ao criar tabela documentos: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_documentos ON documentos(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_documentos: ";
        self.runSqlQuery(sql, errorMessage);
        sql = "CREATE INDEX IF NOT EXISTS idx_documentos_2 ON documentos(codigoPaciente)";
        errorMessage = "CELK Erro ao criar indice idx_documentos_2: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaPacienteEsus = function(){
        var sql = "CREATE TABLE IF NOT EXISTS pacienteEsus ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoSistema INTEGER NULL, " +
                "versao INTEGER NULL, " +
                "codigoPaciente INTEGER NOT NULL,"+
                "dataCadastro INTEGER NOT NULL,"+
                "situacaoConjugal INTEGER,"+
                "codigoCbo INTEGER,"+
                "frequentaEscola INTEGER,"+
                "nivelEscolaridade INTEGER,"+
                "situacaoMercadoTrabalho INTEGER,"+
                "responsavelCrianca INTEGER,"+
                "frequentaCurandeira INTEGER,"+
                "participaGrupoComunitario INTEGER,"+
                "possuiPlanoSaude INTEGER,"+
                "membroComunidadeTradicional INTEGER,"+
                "comunidadeTradicional VARCHAR(50),"+
                "orientacaoSexual INTEGER,"+
                "deficienciaAuditiva INTEGER,"+
                "deficienciaVisual INTEGER,"+
                "deficienciaFisica INTEGER,"+
                "deficienciaIntelectual INTEGER,"+
                "deficienciaOutra INTEGER,"+
                "situacaoRua INTEGER,"+
                "tempoRua INTEGER,"+
                "acompanhadoPorOutraInstituicao INTEGER,"+
                "nomeOutraInstituicao VARCHAR(150),"+
                "recebeBeneficio INTEGER,"+
                "possuiReferenciaFamiliar INTEGER,"+
                "visitaFamiliarFrequentemente INTEGER,"+
                "grauParentesco VARCHAR(50),"+
                "refeicoesDia INTEGER,"+
                "refeicaoRestaurantePopular INTEGER,"+
                "refeicaoDoacaoRestaurante INTEGER,"+
                "refeicaoDoacaoReligioso INTEGER,"+
                "refeicaoDoacaoPopular INTEGER,"+
                "refeicaoDoacaoOutros INTEGER,"+
                "acessoHigienePessoal INTEGER,"+
                "higieneBanho INTEGER,"+
                "higieneSanitario INTEGER,"+
                "higieneBucal INTEGER,"+
                "higieneOutros INTEGER,"+
                "estaGestante INTEGER,"+
                "maternidadeReferencia VARCHAR(150),"+
                "pesoConsiderado INTEGER,"+
                "fumante INTEGER,"+
                "dependenteAlcool INTEGER,"+
                "dependenteDroga INTEGER,"+
                "temHipertensao INTEGER,"+
                "temDiabetes INTEGER,"+
                "teveAvc INTEGER,"+
                "teveInfarto INTEGER,"+
                "temHanseniase INTEGER,"+
                "temTuberculose INTEGER,"+
                "temTeveCancer INTEGER,"+
                "internacaoAno INTEGER,"+
                "causaInternacao VARCHAR(150),"+
                "fezTratamentoPsiquiatrico INTEGER,"+
                "estaAcamado INTEGER,"+
                "estaDomiciliado INTEGER,"+
                "usaPlantasMedicinais INTEGER,"+
                "quaisPlantas VARCHAR(150),"+
                "doencaCardiaca INTEGER,"+
                "cardiacaInsuficiencia INTEGER,"+
                "cardiacaOutros INTEGER,"+
                "cardiacaNaoSabe INTEGER,"+
                "doencaRins INTEGER,"+
                "rinsInsuficiencia INTEGER,"+
                "rinsOutros INTEGER,"+
                "rinsNaoSabe INTEGER,"+
                "doencaRespiratoria INTEGER,"+
                "respiratoriaAsma INTEGER,"+
                "respiratoriaEfisema INTEGER,"+
                "respiratoriaOutros INTEGER,"+
                "respiratoriaNaoSabe INTEGER,"+
                "outrasPraticasIntegrativas INTEGER,"+
                "condicaoSaude1 VARCHAR(100),"+
                "condicaoSaude2 VARCHAR(100),"+
                "condicaoSaude3 VARCHAR(100),"+
                "possuiDeficiencia INTEGER,"+
                "informaOrientacaoSexual INTEGER,"+
                "possuiSofrimentoPsiquicoGrave INTEGER,"+
                "utilizaProtese INTEGER,"+
                "proteseAuditiva INTEGER,"+
                "proteseMembrosSuperiores INTEGER,"+
                "proteseMembrosInferiores INTEGER,"+
                "proteseCadeiraRodas INTEGER,"+
                "proteseOutros INTEGER,"+
                "codigoUsuario INTEGER,"+
                "versaoMobile INTEGER DEFAULT 0,"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id),"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id),"+
                "FOREIGN KEY(codigoCbo) REFERENCES cbo(id))";
        var errorMessage = "CELK Erro ao criar tabela pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_pacienteEsus ON pacienteEsus(codigoSistema)";
        errorMessage = "CELK Erro ao criar indice idx_pacienteEsus: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_pacienteEsus_2 ON pacienteEsus(codigoPaciente)";
        errorMessage = "CELK Erro ao criar indice idx_pacienteEsus_2: ";
        self.runSqlQuery(sql, errorMessage);
        
        sql = "CREATE INDEX IF NOT EXISTS idx_pacienteEsus_3 ON pacienteEsus(versaoMobile)";
        errorMessage = "CELK Erro ao criar indice idx_pacienteEsus_3: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaVisitaDomiciliar = function(){
        var sql = "CREATE TABLE IF NOT EXISTS visitaDomiciliar ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoUsuario INTEGER NOT NULL, " +
                "codigoDomicilio INTEGER NOT NULL, " +
                "dataVisita INTEGER NOT NULL,"+
                "compartilhada INTEGER NOT NULL,"+
                "codigoPaciente INTEGER NOT NULL,"+
                "desfecho INTEGER NOT NULL,"+
                "motivoAtualizacao INTEGER,"+
                "motivoVisitaPeriodica INTEGER,"+
                "motivoBaConsulta INTEGER,"+
                "motivoBaExame INTEGER,"+
                "motivoBaVacina INTEGER,"+
                "motivoBaCondicionalidadesBolsaFamilia INTEGER,"+
                "motivoAcGestante INTEGER,"+
                "motivoAcPuerpera INTEGER,"+
                "motivoAcRecemNascido INTEGER,"+
                "motivoAcCrianca INTEGER,"+
                "motivoAcPessoaDesnutrida INTEGER,"+
                "motivoAcPessoaDeficiente INTEGER,"+
                "motivoAcPessoaHipertensa INTEGER,"+
                "motivoAcPessoaDiabetes INTEGER,"+
                "motivoAcPessoaAsmatica INTEGER,"+
                "motivoAcPessoaEnfisema INTEGER,"+
                "motivoAcPessoaCancer INTEGER,"+
                "motivoAcPessoaDoencaCronica INTEGER,"+
                "motivoAcPessoaHanseniase INTEGER,"+
                "motivoAcPessoaTuberculose INTEGER,"+
                "motivoAcAcamado INTEGER,"+
                "motivoAcVulnerabilidadeSocial INTEGER,"+
                "motivoAcCondicionalidadesBolsaFamilia INTEGER,"+
                "motivoAcSaudeMental INTEGER,"+
                "motivoAcUsuarioAlcool INTEGER,"+
                "motivoAcUsuarioDroga INTEGER,"+
                "motivoEgressoInternacao INTEGER,"+
                "motivoControleAmbientesVetores INTEGER,"+
                "motivoCampanhaSaude INTEGER,"+
                "motivoPrevencao INTEGER,"+
                "motivoOutros INTEGER,"+
                "motivoTratamentoFocal VARCHAR, "+
                "dataColetaGps INTEGER,"+
                "horaColetaGps INTEGER,"+
                "latitude VARCHAR(50),"+
                "longitude VARCHAR(50),"+
                "versaoMobile INTEGER,"+
                "FOREIGN KEY(codigoUsuario) REFERENCES usuarios(id)"+
                "FOREIGN KEY(codigoDomicilio) REFERENCES domicilio(id)"+
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id))";
        var errorMessage = "CELK Erro ao criar tabela visitaDomiciliar: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaErroDomicilio = function(){
        var sql = "CREATE TABLE IF NOT EXISTS erroDomicilio ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoDomicilio INTEGER NOT NULL, " +
                "mensagem VARCHAR, " +
                "FOREIGN KEY(codigoDomicilio) REFERENCES domicilio(id))";
        var errorMessage = "CELK Erro ao criar tabela erroDomicilio: ";
        self.runSqlQuery(sql, errorMessage)

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_erroDomicilio ON erroDomicilio(codigoDomicilio)";
        errorMessage = "CELK Erro ao criar indice idx_erroDomicilio: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.tabelaErroPaciente = function(){
        var sql = "CREATE TABLE IF NOT EXISTS erroPaciente ( " +
                "id INTEGER PRIMARY KEY AUTOINCREMENT, " +
                "codigoPaciente INTEGER NOT NULL, " +
                "mensagem VARCHAR, " +
                "FOREIGN KEY(codigoPaciente) REFERENCES paciente(id))";
        var errorMessage = "CELK Erro ao criar tabela erroPaciente: ";
        self.runSqlQuery(sql, errorMessage);

        sql = "CREATE UNIQUE INDEX IF NOT EXISTS idx_erroPaciente ON erroPaciente(codigoPaciente)";
        errorMessage = "CELK Erro ao criar indice idx_erroPaciente: ";
        self.runSqlQuery(sql, errorMessage);
    };

    self.executeTransaction = function(tx, query, bindings) {
        bindings = typeof bindings !== 'undefined' ? bindings : [];
        var deferred = $q.defer();

        tx.executeSql(query, bindings, function(transaction, result) {
            deferred.resolve(transaction, result);
        }, function(transaction, error) {
            deferred.reject(error);
            return true;
        });

        return deferred.promise;
    };

    self.query = function(query, bindings) {
        bindings = typeof bindings !== 'undefined' ? bindings : [];
        var deferred = $q.defer();

        self.session.transaction(function(transaction) {
            transaction.executeSql(query, bindings, function(transaction, result) {
                deferred.resolve(result);
            }, function(transaction, error) {
                deferred.reject(error);
            });
        }, function(err){
            deferred.reject(err);
        });

        return deferred.promise;
    };

    self.fetchAll = function(result) {
        var output = [];

        for (var i = 0; i < result.rows.length; i++) {
            output.push(result.rows.item(i));
        }

        return output;
    };

    self.fetch = function(result) {
        if(result.rows.length > 0){
            return result.rows.item(0);
        }else {
            return null;
        }
    };

    return self;
});
