services.factory('importacaoService', function ($rootScope, $q, $http, $state, $resource, DB, FS, storageService, dataService, parsersService, messageService, dataService) {
    var importacaoService = this;

    importacaoService.init = function () {
        $http.get('versao.json').success(function (data) {
            delete dataService.data.voltar;
            dataService.data.erros = [];
            importacaoService.cbo();
            var versaoLocal = data.versaoAtual.replace(/(\r\n|\n|\r)/gm, "");
            storageService.setItem("versaoLocal", versaoLocal);
        });
    };

    importacaoService.cbo = function () {
        importacaoService.importarItem("cbo", "profissional");
    }; 

    importacaoService.profissional = function () {
        importacaoService.importarItem("profissional", "usuarios");
    };

    importacaoService.usuarios = function () {
        importacaoService.importarItem("usuarios", "orgaoEmissor");
    };

    importacaoService.orgaoEmissor = function () {
        importacaoService.importarItem("orgaoEmissor", "raca", "orgao_emissor");
    };

    importacaoService.raca = function () {
        importacaoService.importarItem("raca", "etniaIndigena");
    };

    importacaoService.etniaIndigena = function () {
        importacaoService.importarItem("etniaIndigena", "tipoLogradouro", "etnia_indigena");
    };

    importacaoService.tipoLogradouro = function () {
        importacaoService.importarItem("tipoLogradouro", "pais", "tipo_logradouro");
    };

    importacaoService.pais = function () {
        importacaoService.importarItem("pais", "estado");
    };

    importacaoService.estado = function () {
        importacaoService.importarItem("estado", "cidade");
    };

    importacaoService.cidade = function () {
        importacaoService.importarItem("cidade", "cepBrasil");
    };

    importacaoService.cepBrasil = function () {
        importacaoService.versaoExistente('cep_brasil').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            $http.post(dataService.data.url + G_versao + '/' + G_id + '/consultarRecurso?' + 'nomeRecurso=cep_brasil' + '&versao=' + versao + '&codigoProfissional=' + storageService.getItem("codigoProfissional") + "&versaoMobile=" + storageService.getItem("versaoLocal"))
                .success(function (data, status, headers, config) {
                    parsersService.cepBrasil(data, versao === 0)
                        .then(function () {

                            importacaoService.equipe();

                        }, function () {
                            importacaoService.problemaParse();
                        });
                })
                .error(function (data, status, headers, config) {
                    importacaoService.problemaConexao(data);
                });
        });
    };

    importacaoService.equipe = function () {
        importacaoService.importarItem("equipe", "microArea");
    };

    importacaoService.microArea = function () {
        importacaoService.importarItem("microArea", "equipeProfissional", "micro_area");
    };

    importacaoService.equipeProfissional = function () {
        importacaoService.importarItem("equipeProfissional", "endereco", "equipe_profissional");
    };

    importacaoService.endereco = function () {
        importacaoService.versaoExistente('endereco').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Endereco', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'endereco').then(function () {
                dataService.data.log.unshift('Importando endereços');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 2000 === 0) {
                                parsersService.endereco(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.endereco(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.finalizarEndereco();
                                });
                            } else {
                                importacaoService.finalizarEndereco();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.filtroMicroArea = function (recurso, versao) {
        var recursoMetodo = recurso.replace('_', '');
        var localParse = recursoMetodo.charAt(0).toLowerCase() + recursoMetodo.slice(1);
        var localFinalizarRecurso = 'finalizar' + recursoMetodo;

        $http.post(dataService.data.url + G_versao + '/' + G_id + '/consultarRecurso?' + 'nomeRecurso=' + recurso.toLowerCase() + '&versao=' + versao + '&codigoProfissional=' + storageService.getItem("codigoProfissional") + "&versaoMobile=" + storageService.getItem("versaoLocal"))
            .success(function (data, status, headers, config) {
                var itens = [];
                linha = data.split('\n');
                dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' ' + recurso.replace('_', ' ') + '(s)');
                parsersService[localParse](data, versao === 0)
                    .then(function () {
                        importacaoService[localFinalizarRecurso]();
                    }, function () {
                        importacaoService.problemaParse();
                    });
            })
            .error(function (data, status, headers, config) {
                importacaoService.problemaConexao(data);
            });
    };

    importacaoService.finalizarEndereco = function () {
        importacaoService.importarFinalizar('Endereços importados com sucesso!', 'domicilio');
    };

    importacaoService.domicilio = function () {
        importacaoService.versaoExistente('domicilio').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Domicilio', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'domicilio').then(function () {
                dataService.data.log.unshift('Importando domicílios');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 2000 === 0) {
                                parsersService.domicilio(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.domicilio(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.finalizarDomicilio();
                                });
                            } else {
                                importacaoService.finalizarDomicilio();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarDomicilio = function () {
        importacaoService.importarFinalizar('Domicílios importados com sucesso!', 'domicilioEsus');
    };

    importacaoService.domicilioEsus = function () {
        importacaoService.versaoExistente('domicilioEsus').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Domicilio_Esus', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'domicilio_esus', dataService.data.bind.domicilioEsus).then(function () {
                dataService.data.log.unshift('Importando Domicílios e-SUS');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 2000 === 0) {
                                parsersService.domicilioEsus(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.domicilioEsus(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.finalizarDomicilioEsus();
                                });
                            } else {
                                importacaoService.finalizarDomicilioEsus();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarDomicilioEsus = function () {
        importacaoService.importarFinalizar('Domicílios e-SUS importados com sucesso!', 'paciente');
    };

    importacaoService.paciente = function () {
        importacaoService.versaoExistente('paciente').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Paciente', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'paciente').then(function () {
                dataService.data.log.unshift('Importando paciente(s)');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 1000 === 0) {
                                parsersService.paciente(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.paciente(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.finalizarPaciente();
                                });
                            } else {
                                importacaoService.finalizarPaciente();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarPaciente = function () {
        importacaoService.importarFinalizar('Pacientes importados com sucesso!', 'cns');
    };

    importacaoService.cns = function () {
        importacaoService.versaoExistente('cns').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Cns', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'cns', dataService.data.bind.cns).then(function () {
                dataService.data.log.unshift('Importando CNS(s)');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 2000 === 0) {
                                parsersService.cns(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.cns(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.documentos();
                                });
                            } else {
                                importacaoService.documentos();
                            }
                        });
                        lr.read(file);
                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarCns = function () {
        importacaoService.documentos();
    };

    importacaoService.documentos = function () {
        importacaoService.versaoExistente('documentos').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Documentos', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'documentos', dataService.data.bind.documentos).then(function () {
                dataService.data.log.unshift('Importando Documento(s)');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 2000 === 0) {
                                parsersService.documentos(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.documentos(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.finalizarDocumentos();
                                });
                            } else {
                                importacaoService.finalizarDocumentos();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarDocumentos = function () {
        importacaoService.importarFinalizar('Documentos importados com sucesso!', 'pacienteEsus');
    };

    importacaoService.pacienteEsus = function () {
        importacaoService.versaoExistente('pacienteEsus').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Paciente_Esus', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'paciente_esus', dataService.data.bind.pacienteEsus).then(function () {
                dataService.data.log.unshift('Importando Paciente Esus(s)');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 1000 === 0) {
                                parsersService.pacienteEsus(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.pacienteEsus(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.pacienteDado();
                                });
                            } else {
                                importacaoService.pacienteDado();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarPacienteEsus = function () {
        importacaoService.pacienteDado();
    };

    importacaoService.pacienteDado = function () {
        importacaoService.versaoExistente('pacienteDado').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            if (storageService.getItem("filtrarPor") == 'M') {
                importacaoService.filtroMicroArea('Paciente_Dado', versao);
                return;
            }

            importacaoService.baixarArquivo(versao, 'paciente_dado', dataService.data.bind.pacienteDado).then(function () {
                dataService.data.log.unshift('Importando Paciente Dado(s)');
                dataService.data.log.unshift('Importação por meio de arquivo');

                FS.fileSystem.root.getFile("celk/familias/tabelaTemp.txt", null, function (fileEntry) {
                    fileEntry.file(function (file) {
                        var lr = new LineReader({
                            chunkSize: 600
                        });
                        var totalCount = 0;
                        var lineGroup = '';
                        lr.on('line', function (line, next) {
                            lineGroup += line + '\n';
                            totalCount++;
                            if (totalCount % 2000 === 0) {
                                parsersService.pacienteDado(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('[' + totalCount + '] Importados');
                                    lineGroup = '';
                                    next();
                                }, function () {
                                    lr.abort();
                                });
                            } else {
                                next();
                            }
                        });
                        lr.on('error', function (err) {
                            messageService.entregaErros(messageService.mensagensErro.LER_ARQUIVO, err);
                        });
                        lr.on('end', function () {
                            if (lineGroup !== '') {
                                parsersService.pacienteDado(lineGroup, versao === 0).then(function () {
                                    dataService.data.log.unshift('TOTAL DE REGISTROS: ' + totalCount);
                                    importacaoService.pacienteDado();
                                });
                            } else {
                                importacaoService.pacienteDado();
                            }
                        });
                        lr.read(file);

                    }, importacaoService.failFile);
                }, importacaoService.failFile);

            }, function (error) {
                importacaoService.problemaImportarArquivo(error);
            });
        });
    };

    importacaoService.finalizarPacienteDado = function () {
        importacaoService.notificacaoPaciente();
    };

    importacaoService.notificacaoPaciente = function () {
        importacaoService.importarItem("notificacaoPaciente", "notificacaoAgendas", "notificacao_paciente");
    };

    importacaoService.notificacaoAgendas = function () {
        importacaoService.versaoExistente('notificacaoAgendas').then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            var uri = dataService.data.url + G_versao + '/' + G_id + '/consultarRecurso?' + 'nomeRecurso=notificacao_agenda' + '&versao=' + versao + "&versaoMobile=" + storageService.getItem("versaoLocal");

            if (storageService.getItem("codigoProfissional")) {
                uri += '&codigoProfissional=' + storageService.getItem("codigoProfissional");
            }


            $http.post(uri)
                .success(function (data, status, headers, config) {
                    parsersService.notificacaoAgendas(data, versao === 0)
                        .then(function () {

                            importacaoService.motivoNaoComparecimento();

                        }, function () {
                            importacaoService.problemaParse();
                        });
                })
                .error(function (data, status, headers, config) {
                    importacaoService.problemaConexao(data);
                });
        });
    };

    importacaoService.motivoNaoComparecimento = function () {
        importacaoService.importarItem("motivoNaoComparecimento", "tipoVacina", "motivo_nao_comparecimento");
    };

    importacaoService.tipoVacina = function () {
        importacaoService.importarItem("tipoVacina", "pesquisa", "tipo_vacina");
    };

    importacaoService.pesquisa = function () {
        importacaoService.importarItem("pesquisa", "perguntaPesquisa");
    };

    importacaoService.perguntaPesquisa = function () {
        importacaoService.importarItem("perguntaPesquisa", "perguntaResposta", "pergunta_pesquisa");
    };

    importacaoService.perguntaResposta = function () {
        importacaoService.importarItem("perguntaResposta", "pesquisaPergunta", "pergunta_resposta");
    };

    importacaoService.pesquisaPergunta = function () {
        importacaoService.importarItem("pesquisaPergunta", "pacientesRegistroVacinaAberto", "pesquisa_pergunta");
    };

    importacaoService.pacientesRegistroVacinaAberto = function () {
        $http.get(dataService.data.url + G_versao + '/' + G_id + '/registroVacinasAberto')
            .success(function (data, status, headers, config) {
                if (data.length > 0) {
                    var binding = [];
                    var sql = "update paciente set registroVacina = 1 where codigoSistema in (";
                    for (var i = 0; i < data.length; i++) {
                        binding.push(data[i]);
                        sql += "?,";
                    }
                    sql = sql.substring(0, sql.length - 1) + ")";
                    DB.query(sql, binding);

                    var binding = [];
                    var sql = "update paciente set registroVacina = 0 where registroVacina = 1 AND codigoSistema not in (";
                    for (var i = 0; i < data.length; i++) {
                        binding.push(data[i]);
                        sql += "?,";
                    }
                    sql = sql.substring(0, sql.length - 1) + ")";
                    DB.query(sql, binding);
                }
                importacaoService.registrosExcluidos();
            })
            .error(function (data, status, headers, config) {
                importacaoService.problemaConexao(data);
            });
    };

    importacaoService.registrosExcluidos = function () {
        var versao = storageService.getItem("versaoExclusao");
        if (!versao) {
            versao = 0;
        }

        $http.post(dataService.data.url + G_versao + '/' + G_id + '/consultarRecurso?' + 'nomeRecurso=EXCLUSOES' + '&versao=' + versao + "&versaoMobile=" + storageService.getItem("versaoLocal"))
            .success(function (data, status, headers, config) {

                parsersService.exclusoes(data, versao === 0);
                importacaoService.limparCodigoNullDominio(importacaoService.dominioPaciente())

            })
            .error(function (data, status, headers, config) {
                importacaoService.problemaConexao(data);
            });
    };

    importacaoService.dominioPaciente = function () {
        importacaoService.versaoExistente('dominioPaciente').then(function (versao) {
            if (!versao) {
                versao = 0;
            }
            importacaoService.baixarArquivo(versao, 'dominio_paciente', dataService.data.bind.dominioPaciente).then(function () {
                dataService.data.log.unshift('Importando Domínio Paciente(s)');

                var lines = dataService.objects.dominio_paciente;
                var totalCount = 0;
                var promisesList = [];

                if (lines && lines.length !== 0) {

                    for (var i = 0; i < lines.length; i++) {
                        if (i % 1000 === 0 && i !== 0) {
                            totalCount = i - 1;
                            promisesList.push(importacaoService.callParserDominioPaciente(lines.slice(i - 1000, i), versao, totalCount, lines.length));
                        }
                    }

                    if (totalCount <= lines.length) {
                        promisesList.push(importacaoService.callParserDominioPaciente(lines.slice(totalCount, lines.length), versao));
                    }

                    $q.all(promisesList).then(function () {
                        dataService.data.log.unshift('TOTAL DE REGISTROS: ' + lines.length);
                        dataService.data.log.unshift('Domínio Paciente(s) importado(s) com sucesso!');
                        dataService.data.log.unshift('Limpando códigos órfãos de Domínio Paciente(s)');
                        importacaoService.limparCodigoInternoDominio(null, function () {
                            dataService.data.log.unshift('Erro ao limpar códigos órfãos de Domínio Paciente(s)');
                        });
                        importacaoService.sucessoNaImportacao();
                    });
                } else {
                    dataService.data.log.unshift('Limpando códigos órfãos de Domínio Paciente(s)');
                    importacaoService.limparCodigoInternoDominio(null, function () {
                        dataService.data.log.unshift('Erro ao limpar códigos órfãos de Domínio Paciente(s)');
                    });
                    importacaoService.sucessoNaImportacao();
                }

            }, function (error) {
                messageService.entregaErros(messageService.mensagensErro.IMPORTAR, error);
            });
        });
    };

    importacaoService.callParserDominioPaciente = function (lineGroup, versao, totalCount, totalLength) {
        var defer = $q.defer();

        parsersService.dominioPaciente(lineGroup, versao === 0, totalCount, totalLength).then(function () {
            defer.resolve();
        });

        return defer.promise;
    };

    importacaoService.failFile = function () {
        messageService.entregaErros(messageService.mensagensErro.ARQUIVO);
        importacaoService.erroNaImportacao();
    };

    importacaoService.problemaImportarArquivo = function (error) {
        messageService.entregaErros(messageService.mensagensErro.COMUNICACAO_SERVIDOR);
        importacaoService.erroNaImportacao();
    };

    importacaoService.sucessoNaImportacao = function () {
        DB.createVersionTriggers();
        delete dataService.data.voltar;
        dataService.data.erros = [];
        importacaoService.chamaRespondeu().then(function () {
            dataService.data.log.unshift('PROCESSO DE SINCRONIZAÇÃO FINALIZADO');
            storageService.setItem("sincronizacaoFinalizada", true);
            dataService.data.avancar = true;
            dataService.data.desabilita = false;
            storageService.removeItem("desabilita");
            storageService.setItem("ressincronizar", false);
            dataService.data.processando = false;
            storageService.removeItem("confirmandoLote");
            storageService.removeItem("loteErro");
        }, function () {
            messageService.entregaErros(messageService.mensagensErro.COMUNICACAO_SERVIDOR);
            importacaoService.erroNaImportacao();
        });
    };

    importacaoService.limparCodigoInternoDominio = function (suc, err) {
        var updateSql1 = "UPDATE dominioPaciente set codigoInterno = null where codigoInterno is not null;";
        var updateSql2 = "UPDATE dominioPaciente set codigoInterno = (SELECT id FROM paciente WHERE codigoSistema = dominioPaciente.codigoSistema) " +
        "where exists (SELECT 1 FROM paciente WHERE codigoSistema = dominioPaciente.codigoSistema);";

        DB.query(updateSql1, []).then(function(){
            DB.query(updateSql2, []).then(suc, err);
        }, err);
    };

    importacaoService.importarFinalizar = function (log, nextItem) {
        dataService.data.log.unshift(log);
        importacaoService[nextItem]();
    };

    importacaoService.importarItem = function (currentItem, nextItem, nomeRecurso) {
        importacaoService.versaoExistente(currentItem).then(function (versao) {
            if (!versao) {
                versao = 0;
            }

            $http.post(dataService.data.url + G_versao + '/' + G_id + '/consultarRecurso?' + 'nomeRecurso=' + (nomeRecurso || currentItem) + '&versao=' + versao + "&versaoMobile=" + storageService.getItem("versaoLocal"))
                .success(function (data, status, headers, config) {
                    parsersService[currentItem](data, versao === 0)
                        .then(function () {

                            importacaoService[nextItem]();

                        }, function () {
                            importacaoService.problemaParse();
                        });
                })
                .error(function (data, status, headers, config) {
                    importacaoService.problemaConexao(data);
                });
        });
    };

    importacaoService.problemaParse = function () {
        importacaoService.erroNaImportacao();
    };

    importacaoService.problemaConexao = function (data) {
        if (data.data) {
            messageService.entregaErros(messageService.mensagensErro.SINCRONIZACAO, data.data);
        } else {
            messageService.entregaErros(messageService.mensagensErro.COMUNICACAO_SERVIDOR);
        }
        dataService.data.processando = false;
        importacaoService.erroNaImportacao();
    };

    importacaoService.erroNaImportacao = function () {
        DB.createVersionTriggers();

        if (dataService.data.usuarioComPermissao) {
            dataService.data.nova = true;
        }
        dataService.data.continuar = true;
    };

    importacaoService.versaoExistente = function (tabela) {
        var deferred = $q.defer();
        DB.session.transaction(function (tx) {
            var query = 'SELECT max(versao) AS versao FROM ' + tabela;
            tx.executeSql(query, [], function (transaction, result) {
                deferred.resolve(DB.fetch(result).versao);
            }, function (transaction, error) {
                deferred.reject(error);
                return true;
            });
        });
        return deferred.promise;
    };

    importacaoService.baixarArquivo = function (versao, nomeRecurso, versaoFinal) {
        var deferred = $q.defer();

        var uri = dataService.data.url + G_versao + '/' + G_id + '/consultarRecurso?' + 'nomeRecurso=' + nomeRecurso + '&versao=' + versao + "&versaoMobile=" + storageService.getItem("versaoLocal");

        if (storageService.getItem("codigoProfissional")) {
            uri += '&codigoProfissional=' + storageService.getItem("codigoProfissional");
        }

        if (isValid(versaoFinal)) {
            uri += '&versaoFinal=' + versaoFinal;
        }

        uri = encodeURI(uri);

        var _downloadFile = function (url, success, error, onProgress) {
            var xhr = new XMLHttpRequest();
            xhr.open('POST', url, true);
            xhr.onreadystatechange = function () {
                if (xhr.readyState === 4) {
                    if (xhr.status === 200 && success) {
                        success(xhr.response)
                    } else if (error) {
                        error(xhr.response)
                    }
                }
            };
            xhr.addEventListener("progress", onProgress, false);
            xhr.send(null);
        }

        var currentDownloaded;
        _downloadFile(uri, function (data) {
            dataService.data.log.unshift('Gravando arquivo: ' + nomeRecurso);
            $rootScope.$apply();
            dataService.objects[nomeRecurso] = angular.copy(data).split('\n');
            deferred.resolve();

        }, function (error) {
            importacaoService.problemaImportarArquivo(error);
            deferred.reject(error);
        }, function (evt) {
            if (evt.lengthComputable) {
                dataService.data.log.shift();
                $rootScope.$apply();
                var percentComplete = (evt.loaded / evt.total) * 100;
                dataService.data.log.unshift('Baixando arquivo: ' + nomeRecurso + ' (' + percentComplete + ' % concluído)');
                $rootScope.$apply();
            } else {
                var _localDownloaded = parseFloat(evt.loaded / 1024 / 1024, 10).toFixed(2);
                if (currentDownloaded !== _localDownloaded) {
                    dataService.data.log.shift();
                    $rootScope.$apply()
                    currentDownloaded = _localDownloaded
                    dataService.data.log.unshift('Baixando arquivo: ' + nomeRecurso + ' (' + currentDownloaded + ' MB)');
                    $rootScope.$apply();
                }
            }
        });

        return deferred.promise;
    };

    importacaoService.limparCodigoNullDominio = function () {
        DB.query("DELETE FROM dominioPaciente WHERE codigoSistema is null;", []).then(function () {});
    };

    importacaoService.chamaRespondeu = function () {

        delete dataService.data.voltar;
        dataService.data.erros = [];
        var _q = $q.defer();
        if (!storageService.getItem("confirmandoLote")) {
            // Controla para que o recurso de receber reposta do backend rode um por vez.
            storageService.setItem("confirmandoLote", true);

            var _url2 = dataService.data.url + G_versao + '/' + G_id + '/respondeu';

            if (dataService.data.recebi.length <= 0) {
                dataService.data.log.unshift('LOTE NÃO INFORMADO NA CONFIRMAÇÃO DO LOTE');
                _q.resolve();
            } else {
                dataService.data.log.unshift('CONFIRMANDO O RECEBIMENTO DO LOTE ' + dataService.data.recebi);
                return $http.post(_url2, dataService.data.recebi);
            }
        } else {
            if (_q) {
                _q.resolve();
            } else {
                _q.resolve();
            }

        }
        return _q.promise;
    };

    return importacaoService;
});