services.factory('parsersService', function($rootScope, $q, $http, $state, $resource, DB, FS, storageService, dataService, messageService) {
    var parsersService = this;

    function _mostrarErro(itemError, errorMessage, e){
        if(e && e.message){
            dataService.data.erros.unshift(e.message);
        }else{
            dataService.data.erros.unshift(e);
        }
        dataService.data.erros.unshift(errorMessage);
        dataService.data.erros.unshift(" ");
        messageService.entregaErros(messageService.mensagensErro.IMPORTAR);
        messageService.entregaErros(messageService.mensagensErro.SUPORTE);
    }

    parsersService.cbo = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'CBO(s)', 'cbo', 'CBOs', 'CBOs', [ 
            {name: "descricao", type: "emptyToNull"}
        ]);
    };

    parsersService.profissional = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Profissional(s)', 'profissional', 'profissionais', 'Profissionais', [ 
            {name: "nome", type: "emptyToNull"}
        ]);
    };

    parsersService.usuarios = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Usuário(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "nome": emptyToNull(registro[2]),
                "login": emptyToNull(registro[3]),
                "senha": emptyToNull(registro[4]),
                "status": emptyToNull(registro[5]),
                "codigoProfissional": parseInt(registro[6]),
                "nivel": emptyToNull(registro[7])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                if (novo) {
                    var sql = "INSERT OR REPLACE INTO usuarios " +
                        "(id, codigoSistema, versao, nome, login, senha, status, codigoProfissional, nivel) " +
                        "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)";
                    if (!u.codigoProfissional) {
                        for (var key in u) {
                            if (u.hasOwnProperty(key)) {
                                binding.push(u[key]);
                            }
                        }
                    } else {
                        sql = "INSERT OR REPLACE INTO usuarios " +
                            "(id, codigoSistema, versao, nome, login, senha, status, codigoProfissional, nivel) " +
                            " SELECT " + u.id + "," + u.codigoSistema + "," + u.versao + ",\"" + u.nome + "\",\"" + u.login + "\",\"" + u.senha + "\",\"" + u.status + "\", id,\"" + u.nivel + "\"" +
                            " FROM profissional" +
                            " WHERE codigoSistema = ?";
                        binding.push(u.codigoProfissional);
                    }
                } else {
                    if (!u.codigoProfissional) {
                        sql = "INSERT OR REPLACE INTO usuarios " +
                            "(id, codigoSistema, versao, nome, login, senha, status, codigoProfissional, nivel) " +
                            "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                            u.codigoSistema + "," + u.versao + ",\"" + u.nome + "\",\"" + u.login + "\",\"" + u.senha + "\",\"" + u.status + "\", null ,\"" + u.nivel + "\"" +
                            " FROM usuarios WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM usuarios WHERE codigoSistema = ?) LIMIT 1";
                    } else {
                        sql = "INSERT OR REPLACE INTO usuarios " +
                            "(id, codigoSistema, versao, nome, login, senha, status, codigoProfissional, nivel) " +
                            "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                            u.codigoSistema + "," + u.versao + ",\"" + u.nome + "\",\"" + u.login + "\",\"" + u.senha + "\",\"" + u.status + "\", (SELECT id FROM profissional WHERE codigoSistema = " + u.codigoProfissional + ") ,\"" + u.nivel + "\"" +
                            " FROM usuarios WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM usuarios WHERE codigoSistema = ?) LIMIT 1";
                    }
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    if (result.rowsAffected !== 1) {
                        errorMessage = 'Código do profissional não encontrado';
                        transaction.rollback();
                    }
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("usuários", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Usuários importados com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    };

    parsersService.orgaoEmissor = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Orgão(s) de Emissão', 'orgaoEmissor', 'Orgãos de Emissão', 'Orgãos de Emissão', [ 
            {name: "descricao", type: "none"},
            {name: "sigla", type: "emptyToNull"}
        ]);
    };

    parsersService.raca = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Raça(s)', 'raca', 'Raças', 'Raças', [
            {name: "descricao", type: "emptyToNull"},
        ]);
    };

    parsersService.etniaIndigena = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Etnia(s)', 'etniaIndigena', 'Etnias Indigenas', 'Etnias Indigenas', [
            {name: "descricao", type: "emptyToNull"},
        ]);
    }

    parsersService.tipoLogradouro = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Tipo(s) de Logradouro', 'tipoLogradouro', 'Tipos de Logradouro', 'Tipos de Logradouro', [
            {name: "descricao", type: "emptyToNull"},
            {name: "sigla", type: "emptyToNull"},
            {name: "ordem", type: "value", value: 999},
        ]);
    }

    parsersService.pais = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'País(es)', 'pais', 'Países', 'Países', [
            {name: "descricao", type: "emptyToNull"},
            {name: "ordem", type: "value", value: 999},
        ]);
    }

    parsersService.estado = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Estado(s)', 'estado', 'Estado', 'Estados', [
            {name: "descricao", type: "emptyToNull"},
            {name: "sigla", type: "emptyToNull"},
        ]);
    }

    parsersService.cidade = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Cidade(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "descricao": emptyToNull(registro[2]),
                "codigoEstado": emptyToNull(registro[3])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    var sql = "INSERT OR REPLACE INTO cidade " +
                        "(id, codigoSistema, versao, descricao, codigoEstado) " +
                        "VALUES (?, ?, ?, ?, ?)";
                    if (!u.codigoEstado) {
                        for (var key in u) {
                            if (u.hasOwnProperty(key)) {
                                binding.push(u[key]);
                            }
                        }
                    } else {
                        sql = "INSERT OR REPLACE INTO cidade " +
                            "(id, codigoSistema, versao, descricao, codigoEstado) " +
                            " SELECT " + u.id + "," + u.codigoSistema + "," + u.versao + ",\"" + u.descricao + "\", id" +
                            " FROM estado" +
                            " WHERE codigoSistema = ?";
                        binding.push(u.codigoEstado);
                    }
                } else {
                    if (!u.codigoEstado) {
                        sql = "INSERT OR REPLACE INTO cidade " +
                            "(id, codigoSistema, versao, descricao, codigoEstado) " +
                            "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                            u.codigoSistema + "," + u.versao + ",\"" + u.descricao + "\", null" +
                            " FROM cidade WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM cidade WHERE codigoSistema = ?) LIMIT 1";
                    } else {
                        sql = "INSERT OR REPLACE INTO cidade " +
                            "(id, codigoSistema, versao, descricao, codigoEstado) " +
                            "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                            u.codigoSistema + "," + u.versao + ",\"" + u.descricao + "\", (SELECT id FROM estado WHERE codigoSistema = " + u.codigoEstado + ")" +
                            " FROM cidade WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM cidade WHERE codigoSistema = ?) LIMIT 1";
                    }
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 1000 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Cidade", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Cidades importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.cepBrasil = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' CEP');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[2]),
                "cep": emptyToNull(registro[1])
                
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO cep_brasil " +
                        "(id, codigoSistema, versao, cep) " +
                        "VALUES (?, ?, ?, ?)";
                    for (var key in u) {
                        if (u.hasOwnProperty(key)) {
                            binding.push(u[key]);
                        }
                    }
                } else {
                    sql = "INSERT OR REPLACE INTO cep_brasil " +
                        "(id, codigoSistema, versao, cep) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?,? FROM equipe WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM equipe WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.cep);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("CEP", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('CEP importados com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.equipe = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Equipe(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoEmpresa": emptyToNull(registro[2]),
                "descricao": emptyToNull(registro[3])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO equipe " +
                        "(id, codigoSistema, versao, codigoEmpresa, descricao) " +
                        "VALUES (?, ?, ?, ?, ?)";
                    for (var key in u) {
                        if (u.hasOwnProperty(key)) {
                            binding.push(u[key]);
                        }
                    }
                } else {
                    sql = "INSERT OR REPLACE INTO equipe " +
                        "(id, codigoSistema, versao, codigoEmpresa, descricao) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?,?,? FROM equipe WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM equipe WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoEmpresa);
                    binding.push(u.descricao);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Equipe", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Equipes importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.microArea = function(data, novo) {

        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Micro Área(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "microArea": parseInt(registro[2])
                    //                    "equipeProfissional": parseInt(registro[3])//campo não utilizado atualmente
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO microArea " +
                        "(id, codigoSistema, versao, microArea) " +
                        "VALUES (?, ?, ?, ?)";
                    for (var key in u) {
                        if (u.hasOwnProperty(key)) {
                            binding.push(u[key]);
                        }
                    }
                } else {
                    sql = "INSERT OR REPLACE INTO microArea " +
                        "(id, codigoSistema, versao, microArea) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?,? FROM microArea WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM microArea WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.microArea);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Micro Área", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Micro áreas importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.equipeProfissional = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Equipe(s) Profissional');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoProfissional": parseInt(registro[2]),
                "codigoMicroArea": parseInt(registro[3]),
                "codigoEquipe": parseInt(registro[4]),
                "status": parseInt(registro[5])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO equipeProfissional " +
                        "(id, codigoSistema, versao, codigoProfissional, codigoMicroArea, codigoEquipe, status) " +
                        " VALUES (?, ?, ?," +
                        "(SELECT id FROM profissional WHERE codigoSistema = ?)," +
                        "(SELECT id FROM microArea WHERE codigoSistema = ?)," +
                        "(SELECT id FROM equipe WHERE codigoSistema = ?)," +
                        " ?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoProfissional);
                    binding.push(u.codigoMicroArea);
                    binding.push(u.codigoEquipe);
                    binding.push(u.status);
                } else {
                    sql = "INSERT OR REPLACE INTO equipeProfissional " +
                        "(id, codigoSistema, versao, codigoProfissional, codigoMicroArea, codigoEquipe, status) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?," +
                        "(SELECT id FROM profissional WHERE codigoSistema = ?)," +
                        "(SELECT id FROM microArea WHERE codigoSistema = ?)," +
                        "(SELECT id FROM equipe WHERE codigoSistema = ?)," +
                        "?" +
                        " FROM equipeProfissional WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM equipeProfissional WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoProfissional);
                    binding.push(u.codigoMicroArea);
                    binding.push(u.codigoEquipe);
                    binding.push(u.status);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Equipe Profissional", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Equipe Profissional importado com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.endereco = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoCidade": parseInt(registro[2]),
                "cep": emptyToNull(registro[3]),
                "bairro": emptyToNull(registro[4]),
                "codigoTipoLogradouro": parseInt(registro[5]),
                "logradouro": emptyToNull(registro[6]),
                "complementoLogradouro": emptyToNull(registro[7]),
                "numeroLogradouro": emptyToNull(registro[8]),
                "telefone": emptyToNull(registro[9]),
                "telefoneReferencia": emptyToNull(registro[10]),
                "pontoReferencia": emptyToNull(registro[11])
            };
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO endereco " +
                        "(id, codigoSistema, versao, codigoCidade, cep, bairro, codigoTipoLogradouro, logradouro, complementoLogradouro, numeroLogradouro, telefone, telefoneReferencia, pontoReferencia) " +
                        " VALUES (?, ?, ?," +
                        "(SELECT id FROM cidade WHERE codigoSistema = ?)," +
                        "?,?," +
                        "(SELECT id FROM tipoLogradouro WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoCidade);
                    binding.push(u.cep);
                    binding.push(u.bairro);
                    binding.push(u.codigoTipoLogradouro);
                    binding.push(u.logradouro);
                    binding.push(u.complementoLogradouro);
                    binding.push(u.numeroLogradouro);
                    binding.push(u.telefone);
                    binding.push(u.telefoneReferencia);
                    binding.push(u.pontoReferencia);
                } else {
                    sql = "INSERT OR REPLACE INTO endereco " +
                        "(id, codigoSistema, versao, codigoCidade, cep, bairro, codigoTipoLogradouro, logradouro, complementoLogradouro, numeroLogradouro, telefone, telefoneReferencia, pontoReferencia) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?," +
                        "(SELECT id FROM cidade WHERE codigoSistema = ?)," +
                        "?,?," +
                        "(SELECT id FROM tipoLogradouro WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?" +
                        " FROM endereco WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM endereco WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoCidade);
                    binding.push(u.cep);
                    binding.push(u.bairro);
                    binding.push(u.codigoTipoLogradouro);
                    binding.push(u.logradouro);
                    binding.push(u.complementoLogradouro);
                    binding.push(u.numeroLogradouro);
                    binding.push(u.telefone);
                    binding.push(u.telefoneReferencia);
                    binding.push(u.pontoReferencia);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Endereços", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.domicilio = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoEndereco": parseInt(registro[2]),
                "numeroFamilia": parseInt(registro[3]),
                "codigoMicroArea": parseInt(registro[4]),
                "visitaNoMes": parseInt(registro[5])
            };
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO domicilio " +
                        "(id, codigoSistema, versao, codigoEndereco, numeroFamilia, codigoMicroArea, visitaNoMes) " +
                        " VALUES (?, ?, ?," +
                        "(SELECT id FROM endereco WHERE codigoSistema = ?)," +
                        "?," +
                        "(SELECT id FROM microArea WHERE codigoSistema = ?)," +
                        "?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoEndereco);
                    binding.push(u.numeroFamilia);
                    binding.push(u.codigoMicroArea);
                    binding.push(u.visitaNoMes);
                } else {
                    sql = "INSERT OR REPLACE INTO domicilio " +
                        "(id, codigoSistema, versao, codigoEndereco, numeroFamilia, codigoMicroArea, visitaNoMes) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?," +
                        "(SELECT id FROM endereco WHERE codigoSistema = ?)," +
                        "?," +
                        "(SELECT id FROM microArea WHERE codigoSistema = ?)," +
                        "?" +
                        " FROM domicilio WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM domicilio WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoEndereco);
                    binding.push(u.numeroFamilia);
                    binding.push(u.codigoMicroArea);
                    binding.push(u.visitaNoMes);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Domicílios", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.domicilioEsus = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoDomicilio": parseInt(registro[2]),
                "situacaoMoradia": parseInt(registro[3]),
                "localizacao": parseInt(registro[4]),
                "tipoDomicilio": parseInt(registro[5]),
                "numeroMoradores": parseInt(registro[6]),
                "numeroComodos": parseInt(registro[7]),
                "condicaoUsoTerra": parseInt(registro[8]),
                "tipoAcessoDomicilio": parseInt(registro[9]),
                "materialDominante": parseInt(registro[10]),
                "possuiEnergiaEletrica": parseInt(registro[11]),
                "abastecimentoAgua": parseInt(registro[12]),
                "tratamentoAgua": parseInt(registro[13]),
                "esgotamento": parseInt(registro[14]),
                "destinoLixo": parseInt(registro[15]),
                "gato": parseInt(registro[16]),
                "cachorro": parseInt(registro[17]),
                "passaro": parseInt(registro[18]),
                "criacao": parseInt(registro[19]),
                "outros": parseInt(registro[20]),
                "quantos": parseInt(registro[21]),
                "dataColetaGps": null,
                "latitude": emptyToNull(registro[24]),
                "longitude": emptyToNull(registro[25]),

                "tipoImovel": parseInt(registro[26]),
                "nomeInstituicao": emptyToNull(registro[27]),
                "flagOutrosProfissionais": emptyToNull(registro[28]),
                "nomeResponsavelTecnico": emptyToNull(registro[29]),
                "numeroCartaoResponsavelTecnico": emptyToNull(registro[30]),
                "cargoResponsavelTecnico": emptyToNull(registro[31]),
                "telefoneResponsavelTecnico": emptyToNull(registro[32])
                // "flagForaArea": emptyToNull(registro[26])
            };

            if (registro[23] !== '') {
                var date = registro[22].split('-');
                var hora = registro[23].split(':');
                itens[i].dataColetaGps = new Date(date[2], date[1] - 1, date[0], hora[0], hora[1], hora[2], 0).getTime() / 1000;
            }
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO domicilioEsus " +
                        "(id, codigoSistema, versao, codigoDomicilio, situacaoMoradia, localizacao, tipoDomicilio, numeroMoradores, numeroComodos," +
                        "condicaoUsoTerra, tipoAcessoDomicilio, materialDominante, possuiEnergiaEletrica, abastecimentoAgua, tratamentoAgua, esgotamento, destinoLixo," +
                        "gato, cachorro, passaro, criacao, outros, quantos, dataColetaGps, latitude, longitude, " +
                        "tipoImovel, nomeInstituicao, flagOutrosProfissionais, nomeResponsavelTecnico, numeroCartaoResponsavelTecnico, cargoResponsavelTecnico, telefoneResponsavelTecnico)"+
                        " VALUES (?, ?, ?," +
                        "(SELECT id FROM domicilio WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?, ?,?,?,?,?,?,?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoDomicilio);
                    binding.push(u.situacaoMoradia);
                    binding.push(u.localizacao);
                    binding.push(u.tipoDomicilio);
                    binding.push(u.numeroMoradores);
                    binding.push(u.numeroComodos);
                    binding.push(u.condicaoUsoTerra);
                    binding.push(u.tipoAcessoDomicilio);
                    binding.push(u.materialDominante);
                    binding.push(u.possuiEnergiaEletrica);
                    binding.push(u.abastecimentoAgua);
                    binding.push(u.tratamentoAgua);
                    binding.push(u.esgotamento);
                    binding.push(u.destinoLixo);
                    binding.push(u.gato);
                    binding.push(u.cachorro);
                    binding.push(u.passaro);
                    binding.push(u.criacao);
                    binding.push(u.outros);
                    binding.push(u.quantos);
                    binding.push(u.dataColetaGps);
                    binding.push(u.latitude);
                    binding.push(u.longitude);

                    binding.push(u.tipoImovel);
                    binding.push(u.nomeInstituicao);
                    binding.push(u.flagOutrosProfissionais);
                    binding.push(u.nomeResponsavelTecnico);
                    binding.push(u.numeroCartaoResponsavelTecnico);
                    binding.push(u.cargoResponsavelTecnico);
                    binding.push(u.telefoneResponsavelTecnico);

                    // binding.push(u.flagForaArea);
                } else {
                    sql = "INSERT OR REPLACE INTO domicilioEsus " +
                        "(id, codigoSistema, versao, codigoDomicilio, situacaoMoradia, localizacao, tipoDomicilio, numeroMoradores, numeroComodos," +
                        "condicaoUsoTerra, tipoAcessoDomicilio, materialDominante, possuiEnergiaEletrica, abastecimentoAgua, tratamentoAgua, esgotamento, destinoLixo," +
                        "gato, cachorro, passaro, criacao, outros, quantos, dataColetaGps, latitude, longitude, " +
                        "tipoImovel, nomeInstituicao, flagOutrosProfissionais, nomeResponsavelTecnico, numeroCartaoResponsavelTecnico, cargoResponsavelTecnico, telefoneResponsavelTecnico)" +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?," +
                        "(SELECT id FROM domicilio WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?"
                        " FROM domicilioEsus WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM domicilioEsus WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoDomicilio);
                    binding.push(u.situacaoMoradia);
                    binding.push(u.localizacao);
                    binding.push(u.tipoDomicilio);
                    binding.push(u.numeroMoradores);
                    binding.push(u.numeroComodos);
                    binding.push(u.condicaoUsoTerra);
                    binding.push(u.tipoAcessoDomicilio);
                    binding.push(u.materialDominante);
                    binding.push(u.possuiEnergiaEletrica);
                    binding.push(u.abastecimentoAgua);
                    binding.push(u.tratamentoAgua);
                    binding.push(u.esgotamento);
                    binding.push(u.destinoLixo);
                    binding.push(u.gato);
                    binding.push(u.cachorro);
                    binding.push(u.passaro);
                    binding.push(u.criacao);
                    binding.push(u.outros);
                    binding.push(u.quantos);
                    binding.push(u.dataColetaGps);
                    binding.push(u.latitude);
                    binding.push(u.longitude);

                    binding.push(u.tipoImovel);
                    binding.push(u.nomeInstituicao);
                    binding.push(u.flagOutrosProfissionais);
                    binding.push(u.nomeResponsavelTecnico);
                    binding.push(u.numeroCartaoResponsavelTecnico);
                    binding.push(u.cargoResponsavelTecnico);
                    binding.push(u.telefoneResponsavelTecnico);

                    // binding.push(u.flagForaArea);

                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Domicílios e-SUS", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.paciente = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "excluido": emptyToNull(registro[2]),
                "nome": emptyToNull(registro[3]),
                "dataNascimento": null,
                "sexo": emptyToNull(registro[5]),
                "nomeMae": emptyToNull(registro[6]),
                "codigoCidade": parseInt(registro[7]),
                "cpf": emptyToNull(registro[8]),
                "rg": emptyToNull(registro[9]),
                "telefone": emptyToNull(registro[10]),
                "celular": emptyToNull(registro[11]),
                "codigoRaca": parseInt(registro[12]),
                "codigoPais": parseInt(registro[13]),
                "email": emptyToNull(registro[14]),
                "apelido": emptyToNull(registro[15]),
                "flagResponsavelFamiliar": parseInt(registro[16]),
                "codigoResponsavelFamiliar": parseInt(registro[17]),
                "nacionalidade": parseInt(registro[18]),
                "codigoDomicilio": parseInt(registro[19]),
                "telefone2": emptyToNull(registro[20]),
                "telefone3": emptyToNull(registro[21]),
                "telefone4": emptyToNull(registro[22]),
                "religiao": emptyToNull(registro[23]),
                "localTrabalho": emptyToNull(registro[24]),
                "telefoneTrabalho": emptyToNull(registro[25]),
                "responsavel": emptyToNull(registro[26]),
                "parentescoResponsavel": emptyToNull(registro[27]),
                "urgenciaNome": emptyToNull(registro[28]),
                "urgenciaTelefone": emptyToNull(registro[29]),
                "urgenciaParentesco": emptyToNull(registro[30]),
                "rendaFamiliar": parseInt(registro[31]),
                "resideDesde": null,
                "situacao": parseInt(registro[33]),
                "nis": parseInt(registro[34]),
                "prontuario": emptyToNull(registro[35]),
                "motivoExclusao": emptyToNull(registro[36]),
                "codigoEtniaIndigena": parseInt(registro[37]),
                "nomePai": emptyToNull(registro[38]),
                "keyword": registro[3] + ' ' + registro[6] + ' ' + registro[8]
            };

            if (registro[4] !== '') {
                var date = registro[4].split('-');
                itens[i].dataNascimento = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
            if (registro[32] !== '') {
                var date = registro[32].split('-');
                itens[i].resideDesde = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO paciente " +
                        "(id, codigoSistema, versao, excluido, nome, dataNascimento, sexo, nomeMae, codigoCidade," +
                        "cpf, rg, telefone, celular, codigoRaca, codigoPais, email, apelido," +
                        "flagResponsavelFamiliar, nacionalidade, codigoDomicilio, telefone2," +
                        "telefone3, telefone4, religiao, localTrabalho, telefoneTrabalho, responsavel, parentescoResponsavel," +
                        "urgenciaNome, urgenciaTelefone, urgenciaParentesco, rendaFamiliar, resideDesde, situacao, nis, prontuario, motivoExclusao, codigoEtniaIndigena, keyword, codigoResponsavelFamiliar, nomePai)" +
                        " VALUES (?, ?, ?," +
                        "?,?,?,?,?," +
                        "(SELECT id FROM cidade WHERE codigoSistema = ?)," +
                        "?,?,?,?," +
                        "(SELECT id FROM raca WHERE codigoSistema = ?)," +
                        "(SELECT id FROM pais WHERE codigoSistema = ?)," +
                        "?,?,?,?," +
                        "(SELECT id FROM domicilio WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?," +
                        "(SELECT id FROM etniaIndigena WHERE codigoSistema = ?)," +
                        "?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?),"+
                        "?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.extend(parsersService.setPropertiesPaciente(u));

                    tx.executeSql(sql, binding, function(transaction, result) {
                        count++;
                        if (count % 100 === 0 && errorMessage === '') {
                        }
                    }, function(transaction, error) {
                        if (errorMessage === '') {
                            errorMessage = error.message;
                        }
                        return true;
                    });

                } else {

                    parsersService.updatePaciente(tx, u);

                }

                if (!isNaN(u.codigoResponsavelFamiliar)) {
                    sql = "INSERT OR REPLACE INTO pacienteResponsavelAux " +
                        "(codigoPacienteSistema, codigoResponsavelSistema)" +
                        "VALUES (?,?)";
                    binding = [];
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoResponsavelFamiliar);

                    tx.executeSql(sql, binding, function(transaction, result) {}, function(transaction, error) {
                        if (errorMessage === '') {
                            errorMessage = error.message;
                        }
                        return true;
                    });
                } else {
                    sql = "DELETE FROM pacienteResponsavelAux " +
                        "WHERE codigoPacienteSistema = ?";
                    binding = [];
                    binding.push(u.codigoSistema);

                    tx.executeSql(sql, binding, function(transaction, result) {}, function(transaction, error) {
                        if (errorMessage === '') {
                            errorMessage = error.message;
                        }
                        return true;
                    });
                }
            }
        }, function(e) {
            _mostrarErro("Pacientes", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.cns = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "excluido": emptyToNull(registro[2]),
                "numeroCartao": emptyToNull(registro[3]),
                "codigoPaciente": parseInt(registro[4])
            };
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO cns " +
                        "(id, codigoSistema, versao, excluido, numeroCartao, codigoPaciente)" +
                        " VALUES (?, ?, ?, ?, ?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?))";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.excluido);
                    binding.push(u.numeroCartao);
                    binding.push(u.codigoPaciente);
                } else {
                    sql = "INSERT OR REPLACE INTO cns " +
                        "(id, codigoSistema, versao, excluido, numeroCartao, codigoPaciente)" +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?,?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)" +
                        " FROM cns WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM cns WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.excluido);
                    binding.push(u.numeroCartao);
                    binding.push(u.codigoPaciente);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("CNS", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.documentos = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "excluido": emptyToNull(registro[2]),
                "tipoDocumento": parseInt(registro[3]),
                "numeroDocumento": emptyToNull(registro[4]),
                "complemento": emptyToNull(registro[5]),
                "codigoOrgaoEmissor": parseInt(registro[6]),
                "numeroCartorio": emptyToNull(registro[7]),
                "numeroLivro": emptyToNull(registro[8]),
                "numeroFolha": emptyToNull(registro[9]),
                "numeroTermo": emptyToNull(registro[10]),
                "numeroMatricula": emptyToNull(registro[11]),
                "dataEmissao": null,
                "siglaUf": emptyToNull(registro[13]),
                "codigoPaciente": parseInt(registro[14])
            };
            if (registro[12] !== '') {
                var date = registro[12].split('-');
                itens[i].dataEmissao = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO documentos " +
                        "(id, codigoSistema, versao, excluido, tipoDocumento, numeroDocumento, complemento, codigoOrgaoEmissor," +
                        "numeroCartorio, numeroLivro, numeroFolha, numeroTermo, numeroMatricula, dataEmissao, siglaUf, codigoPaciente)" +
                        " VALUES (?, ?, ?, ?, ?,?,?," +
                        "(SELECT id FROM orgaoEmissor WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?))";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.excluido);
                    binding.push(u.tipoDocumento);
                    binding.push(u.numeroDocumento);
                    binding.push(u.complemento);
                    binding.push(u.codigoOrgaoEmissor);
                    binding.push(u.numeroCartorio);
                    binding.push(u.numeroLivro);
                    binding.push(u.numeroFolha);
                    binding.push(u.numeroTermo);
                    binding.push(u.numeroMatricula);
                    binding.push(u.dataEmissao);
                    binding.push(u.siglaUf);
                    binding.push(u.codigoPaciente);
                } else {
                    sql = "INSERT OR REPLACE INTO documentos " +
                        "(id, codigoSistema, versao, excluido, tipoDocumento, numeroDocumento, complemento, codigoOrgaoEmissor," +
                        "numeroCartorio, numeroLivro, numeroFolha, numeroTermo, numeroMatricula, dataEmissao, siglaUf, codigoPaciente)" +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?,?,?,?,?," +
                        "(SELECT id FROM orgaoEmissor WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)" +
                        " FROM documentos WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM documentos WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.excluido);
                    binding.push(u.tipoDocumento);
                    binding.push(u.numeroDocumento);
                    binding.push(u.complemento);
                    binding.push(u.codigoOrgaoEmissor);
                    binding.push(u.numeroCartorio);
                    binding.push(u.numeroLivro);
                    binding.push(u.numeroFolha);
                    binding.push(u.numeroTermo);
                    binding.push(u.numeroMatricula);
                    binding.push(u.dataEmissao);
                    binding.push(u.siglaUf);
                    binding.push(u.codigoPaciente);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Documentos", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.pacienteEsus = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoPaciente": parseInt(registro[2]),
                "dataCadastro": null,
                "situacaoConjugal": parseInt(registro[4]),
                "codigoCbo": parseInt(registro[5]),
                "frequentaEscola": parseInt(registro[6]),
                "nivelEscolaridade": parseInt(registro[7]),
                "situacaoMercadoTrabalho": parseInt(registro[8]),
                "responsavelCrianca": parseInt(registro[9]),
                "frequentaCurandeira": parseInt(registro[10]),
                "participaGrupoComunitario": parseInt(registro[11]),
                "possuiPlanoSaude": parseInt(registro[12]),
                "membroComunidadeTradicional": parseInt(registro[13]),
                "comunidadeTradicional": emptyToNull(registro[14]),
                "orientacaoSexual": parseInt(registro[15]),
                "deficienciaAuditiva": parseInt(registro[16]),
                "deficienciaVisual": parseInt(registro[17]),
                "deficienciaFisica": parseInt(registro[18]),
                "deficienciaIntelectual": parseInt(registro[19]),
                "deficienciaOutra": parseInt(registro[20]),
                "situacaoRua": parseInt(registro[21]),
                "tempoRua": parseInt(registro[22]),
                "acompanhadoPorOutraInstituicao": parseInt(registro[23]),
                "nomeOutraInstituicao": emptyToNull(registro[24]),
                "recebeBeneficio": parseInt(registro[25]),
                "possuiReferenciaFamiliar": parseInt(registro[26]),
                "visitaFamiliarFrequentemente": parseInt(registro[27]),
                "grauParentesco": emptyToNull(registro[28]),
                "refeicoesDia": parseInt(registro[29]),
                "refeicaoRestaurantePopular": parseInt(registro[30]),
                "refeicaoDoacaoRestaurante": parseInt(registro[31]),
                "refeicaoDoacaoReligioso": parseInt(registro[32]),
                "refeicaoDoacaoPopular": parseInt(registro[33]),
                "refeicaoDoacaoOutros": parseInt(registro[34]),
                "acessoHigienePessoal": parseInt(registro[35]),
                "higieneBanho": parseInt(registro[36]),
                "higieneSanitario": parseInt(registro[37]),
                "higieneBucal": parseInt(registro[38]),
                "higieneOutros": parseInt(registro[39]),
                "estaGestante": parseInt(registro[40]),
                "maternidadeReferencia": emptyToNull(registro[41]),
                "pesoConsiderado": parseInt(registro[42]),
                "fumante": parseInt(registro[43]),
                "dependenteAlcool": parseInt(registro[44]),
                "dependenteDroga": parseInt(registro[45]),
                "temHipertensao": parseInt(registro[46]),
                "temDiabetes": parseInt(registro[47]),
                "teveAvc": parseInt(registro[48]),
                "teveInfarto": parseInt(registro[49]),
                "temHanseniase": parseInt(registro[50]),
                "temTuberculose": parseInt(registro[51]),
                "temTeveCancer": parseInt(registro[52]),
                "internacaoAno": parseInt(registro[53]),
                "causaInternacao": emptyToNull(registro[54]),
                "fezTratamentoPsiquiatrico": parseInt(registro[55]),
                "estaAcamado": parseInt(registro[56]),
                "estaDomiciliado": parseInt(registro[57]),
                "usaPlantasMedicinais": parseInt(registro[58]),
                "quaisPlantas": emptyToNull(registro[59]),
                "doencaCardiaca": parseInt(registro[60]),
                "cardiacaInsuficiencia": parseInt(registro[61]),
                "cardiacaOutros": parseInt(registro[62]),
                "cardiacaNaoSabe": parseInt(registro[63]),
                "doencaRins": parseInt(registro[64]),
                "rinsInsuficiencia": parseInt(registro[65]),
                "rinsOutros": parseInt(registro[66]),
                "rinsNaoSabe": parseInt(registro[67]),
                "doencaRespiratoria": parseInt(registro[68]),
                "respiratoriaAsma": parseInt(registro[69]),
                "respiratoriaEfisema": parseInt(registro[70]),
                "respiratoriaOutros": parseInt(registro[71]),
                "respiratoriaNaoSabe": parseInt(registro[72]),
                "outrasPraticasIntegrativas": parseInt(registro[73]),
                "condicaoSaude1": emptyToNull(registro[74]),
                "condicaoSaude2": emptyToNull(registro[75]),
                "condicaoSaude3": emptyToNull(registro[76]),
                "possuiDeficiencia": parseInt(registro[77]),
                "informaOrientacaoSexual": parseInt(registro[78]),
                "possuiSofrimentoPsiquicoGrave": parseInt(registro[79]),
                "utilizaProtese": parseInt(registro[80]),
                "proteseAuditiva": parseInt(registro[81]),
                "proteseMembrosSuperiores": parseInt(registro[82]),
                "proteseMembrosInferiores": parseInt(registro[83]),
                "proteseCadeiraRodas": parseInt(registro[84]),
                "proteseOutros": parseInt(registro[85]),

                "paiDesconhecido": parseInt(registro[87]),
                "maeDesconhecida": parseInt(registro[88]),
                "dataNaturalizado": emptyToNull(registro[89]),
                "portariaNaturalizacao": emptyToNull(registro[90]),
                "dtEntradaBrasil": emptyToNull(registro[91]),
                // "flagForaArea": emptyToNull(registro[91]),
                "dataObito": emptyToNull(registro[92]),
                "numeroDO": emptyToNull(registro[93]),
                "identidadeGenero": parseInt(registro[94])

            };
            if (registro[3] !== '') {
                var date = registro[3].split('-');
                itens[i].dataCadastro = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
            if (registro[89] && registro[89] !== '') {
                var date = registro[89].split('-');
                itens[i].dataNaturalizado = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
            if (registro[91] && registro[91] !== '') {
                var date = registro[91].split('-');
                itens[i].dtEntradaBrasil = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO pacienteEsus " +
                        "(id, codigoSistema, versao, codigoPaciente, dataCadastro, situacaoConjugal, codigoCbo, frequentaEscola, nivelEscolaridade," +
                        "situacaoMercadoTrabalho, responsavelCrianca, frequentaCurandeira, participaGrupoComunitario, possuiPlanoSaude, membroComunidadeTradicional," +
                        "comunidadeTradicional, orientacaoSexual, deficienciaAuditiva, deficienciaVisual, deficienciaFisica, deficienciaIntelectual, deficienciaOutra," +
                        "situacaoRua, tempoRua, acompanhadoPorOutraInstituicao, nomeOutraInstituicao, recebeBeneficio, possuiReferenciaFamiliar," +
                        "visitaFamiliarFrequentemente, grauParentesco, refeicoesDia, refeicaoRestaurantePopular, refeicaoDoacaoRestaurante, refeicaoDoacaoReligioso," +
                        "refeicaoDoacaoPopular, refeicaoDoacaoOutros, acessoHigienePessoal, higieneBanho, higieneSanitario, higieneBucal, higieneOutros, estaGestante," +
                        "maternidadeReferencia, pesoConsiderado, fumante, dependenteAlcool, dependenteDroga, temHipertensao, temDiabetes, teveAvc, teveInfarto," +
                        "temHanseniase, temTuberculose, temTeveCancer, internacaoAno, causaInternacao, fezTratamentoPsiquiatrico, estaAcamado, estaDomiciliado," +
                        "usaPlantasMedicinais, quaisPlantas, doencaCardiaca, cardiacaInsuficiencia, cardiacaOutros, cardiacaNaoSabe, doencaRins, rinsInsuficiencia," +
                        "rinsOutros, rinsNaoSabe, doencaRespiratoria, respiratoriaAsma, respiratoriaEfisema, respiratoriaOutros, respiratoriaNaoSabe," +
                        "outrasPraticasIntegrativas, condicaoSaude1, condicaoSaude2, condicaoSaude3, possuiDeficiencia, informaOrientacaoSexual," +
                        "possuiSofrimentoPsiquicoGrave, utilizaProtese, proteseAuditiva, proteseMembrosSuperiores, proteseMembrosInferiores, proteseCadeiraRodas, proteseOutros, parentescoResponsavel, "+
                        " paiDesconhecido, maeDesconhecida, dataNaturalizacao, portariaNaturalizacao, dtEntradaBrasil, dataObito, numeroDO, identidadeGenero)" +
                        " VALUES (?,?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)," +
                        "?,?," +
                        "(SELECT id FROM cbo WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?,?, ?,?,?,?,?,?,?,?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoPaciente);
                    binding.push(u.dataCadastro);
                    binding.push(u.situacaoConjugal);
                    binding.push(u.codigoCbo);
                    binding.push(u.frequentaEscola);
                    binding.push(u.nivelEscolaridade);
                    binding.push(u.situacaoMercadoTrabalho);
                    binding.push(u.responsavelCrianca);
                    binding.push(u.frequentaCurandeira);
                    binding.push(u.participaGrupoComunitario);
                    binding.push(u.possuiPlanoSaude);
                    binding.push(u.membroComunidadeTradicional);
                    binding.push(u.comunidadeTradicional);
                    binding.push(u.orientacaoSexual);
                    binding.push(u.deficienciaAuditiva);
                    binding.push(u.deficienciaVisual);
                    binding.push(u.deficienciaFisica);
                    binding.push(u.deficienciaIntelectual);
                    binding.push(u.deficienciaOutra);
                    binding.push(u.situacaoRua);
                    binding.push(u.tempoRua);
                    binding.push(u.acompanhadoPorOutraInstituicao);
                    binding.push(u.nomeOutraInstituicao);
                    binding.push(u.recebeBeneficio);
                    binding.push(u.possuiReferenciaFamiliar);
                    binding.push(u.visitaFamiliarFrequentemente);
                    binding.push(u.grauParentesco);
                    binding.push(u.refeicoesDia);
                    binding.push(u.refeicaoRestaurantePopular);
                    binding.push(u.refeicaoDoacaoRestaurante);
                    binding.push(u.refeicaoDoacaoReligioso);
                    binding.push(u.refeicaoDoacaoPopular);
                    binding.push(u.refeicaoDoacaoOutros);
                    binding.push(u.acessoHigienePessoal);
                    binding.push(u.higieneBanho);
                    binding.push(u.higieneSanitario);
                    binding.push(u.higieneBucal);
                    binding.push(u.higieneOutros);
                    binding.push(u.estaGestante);
                    binding.push(u.maternidadeReferencia);
                    binding.push(u.pesoConsiderado);
                    binding.push(u.fumante);
                    binding.push(u.dependenteAlcool);
                    binding.push(u.dependenteDroga);
                    binding.push(u.temHipertensao);
                    binding.push(u.temDiabetes);
                    binding.push(u.teveAvc);
                    binding.push(u.teveInfarto);
                    binding.push(u.temHanseniase);
                    binding.push(u.temTuberculose);
                    binding.push(u.temTeveCancer);
                    binding.push(u.internacaoAno);
                    binding.push(u.causaInternacao);
                    binding.push(u.fezTratamentoPsiquiatrico);
                    binding.push(u.estaAcamado);
                    binding.push(u.estaDomiciliado);
                    binding.push(u.usaPlantasMedicinais);
                    binding.push(u.quaisPlantas);
                    binding.push(u.doencaCardiaca);
                    binding.push(u.cardiacaInsuficiencia);
                    binding.push(u.cardiacaOutros);
                    binding.push(u.cardiacaNaoSabe);
                    binding.push(u.doencaRins);
                    binding.push(u.rinsInsuficiencia);
                    binding.push(u.rinsOutros);
                    binding.push(u.rinsNaoSabe);
                    binding.push(u.doencaRespiratoria);
                    binding.push(u.respiratoriaAsma);
                    binding.push(u.respiratoriaEfisema);
                    binding.push(u.respiratoriaOutros);
                    binding.push(u.respiratoriaNaoSabe);
                    binding.push(u.outrasPraticasIntegrativas);
                    binding.push(u.condicaoSaude1);
                    binding.push(u.condicaoSaude2);
                    binding.push(u.condicaoSaude3);
                    binding.push(u.possuiDeficiencia);
                    binding.push(u.informaOrientacaoSexual);
                    binding.push(u.possuiSofrimentoPsiquicoGrave);
                    binding.push(u.utilizaProtese);
                    binding.push(u.proteseAuditiva);
                    binding.push(u.proteseMembrosSuperiores);
                    binding.push(u.proteseMembrosInferiores);
                    binding.push(u.proteseCadeiraRodas);
                    binding.push(u.proteseOutros);
                    binding.push(u.parentescoResponsavel);

                    binding.push(u.paiDesconhecido);
                    binding.push(u.maeDesconhecida);
                    binding.push(u.dataNaturalizado);
                    binding.push(u.portariaNaturalizacao);
                    binding.push(u.dtEntradaBrasil);
                    binding.push(u.dataObito);
                    binding.push(u.numeroDO);
                    binding.push(u.identidadeGenero);
                } else {
                    sql = "INSERT OR REPLACE INTO pacienteEsus " +
                        "(id, codigoSistema, versao, codigoPaciente, dataCadastro, situacaoConjugal, codigoCbo, frequentaEscola, nivelEscolaridade," +
                        "situacaoMercadoTrabalho, responsavelCrianca, frequentaCurandeira, participaGrupoComunitario, possuiPlanoSaude, membroComunidadeTradicional," +
                        "comunidadeTradicional, orientacaoSexual, deficienciaAuditiva, deficienciaVisual, deficienciaFisica, deficienciaIntelectual, deficienciaOutra," +
                        "situacaoRua, tempoRua, acompanhadoPorOutraInstituicao, nomeOutraInstituicao, recebeBeneficio, possuiReferenciaFamiliar," +
                        "visitaFamiliarFrequentemente, grauParentesco, refeicoesDia, refeicaoRestaurantePopular, refeicaoDoacaoRestaurante, refeicaoDoacaoReligioso," +
                        "refeicaoDoacaoPopular, refeicaoDoacaoOutros, acessoHigienePessoal, higieneBanho, higieneSanitario, higieneBucal, higieneOutros, estaGestante," +
                        "maternidadeReferencia, pesoConsiderado, fumante, dependenteAlcool, dependenteDroga, temHipertensao, temDiabetes, teveAvc, teveInfarto," +
                        "temHanseniase, temTuberculose, temTeveCancer, internacaoAno, causaInternacao, fezTratamentoPsiquiatrico, estaAcamado, estaDomiciliado," +
                        "usaPlantasMedicinais, quaisPlantas, doencaCardiaca, cardiacaInsuficiencia, cardiacaOutros, cardiacaNaoSabe, doencaRins, rinsInsuficiencia," +
                        "rinsOutros, rinsNaoSabe, doencaRespiratoria, respiratoriaAsma, respiratoriaEfisema, respiratoriaOutros, respiratoriaNaoSabe," +
                        "outrasPraticasIntegrativas, condicaoSaude1, condicaoSaude2, condicaoSaude3, possuiDeficiencia, informaOrientacaoSexual," +
                        "possuiSofrimentoPsiquicoGrave, utilizaProtese, proteseAuditiva, proteseMembrosSuperiores, proteseMembrosInferiores, proteseCadeiraRodas, proteseOutros, parentescoResponsavel, "+
                        " paiDesconhecido, maeDesconhecida, dataNaturalizacao, portariaNaturalizacao, dtEntradaBrasil, dataObito, numeroDO, identidadeGenero)" +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)," +
                        "?,?," +
                        "(SELECT id FROM cbo WHERE codigoSistema = ?)," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?," +
                        "?,?,?,?,?,?,?,?,?,?," + "?,?,?,?,?,?,?,?,?,?" +
                        +"?,?,?,?,?,?,?,?,? "+
                        " FROM pacienteEsus WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM pacienteEsus WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoPaciente);
                    binding.push(u.dataCadastro);
                    binding.push(u.situacaoConjugal);
                    binding.push(u.codigoCbo);
                    binding.push(u.frequentaEscola);
                    binding.push(u.nivelEscolaridade);
                    binding.push(u.situacaoMercadoTrabalho);
                    binding.push(u.responsavelCrianca);
                    binding.push(u.frequentaCurandeira);
                    binding.push(u.participaGrupoComunitario);
                    binding.push(u.possuiPlanoSaude);
                    binding.push(u.membroComunidadeTradicional);
                    binding.push(u.comunidadeTradicional);
                    binding.push(u.orientacaoSexual);
                    binding.push(u.deficienciaAuditiva);
                    binding.push(u.deficienciaVisual);
                    binding.push(u.deficienciaFisica);
                    binding.push(u.deficienciaIntelectual);
                    binding.push(u.deficienciaOutra);
                    binding.push(u.situacaoRua);
                    binding.push(u.tempoRua);
                    binding.push(u.acompanhadoPorOutraInstituicao);
                    binding.push(u.nomeOutraInstituicao);
                    binding.push(u.recebeBeneficio);
                    binding.push(u.possuiReferenciaFamiliar);
                    binding.push(u.visitaFamiliarFrequentemente);
                    binding.push(u.grauParentesco);
                    binding.push(u.refeicoesDia);
                    binding.push(u.refeicaoRestaurantePopular);
                    binding.push(u.refeicaoDoacaoRestaurante);
                    binding.push(u.refeicaoDoacaoReligioso);
                    binding.push(u.refeicaoDoacaoPopular);
                    binding.push(u.refeicaoDoacaoOutros);
                    binding.push(u.acessoHigienePessoal);
                    binding.push(u.higieneBanho);
                    binding.push(u.higieneSanitario);
                    binding.push(u.higieneBucal);
                    binding.push(u.higieneOutros);
                    binding.push(u.estaGestante);
                    binding.push(u.maternidadeReferencia);
                    binding.push(u.pesoConsiderado);
                    binding.push(u.fumante);
                    binding.push(u.dependenteAlcool);
                    binding.push(u.dependenteDroga);
                    binding.push(u.temHipertensao);
                    binding.push(u.temDiabetes);
                    binding.push(u.teveAvc);
                    binding.push(u.teveInfarto);
                    binding.push(u.temHanseniase);
                    binding.push(u.temTuberculose);
                    binding.push(u.temTeveCancer);
                    binding.push(u.internacaoAno);
                    binding.push(u.causaInternacao);
                    binding.push(u.fezTratamentoPsiquiatrico);
                    binding.push(u.estaAcamado);
                    binding.push(u.estaDomiciliado);
                    binding.push(u.usaPlantasMedicinais);
                    binding.push(u.quaisPlantas);
                    binding.push(u.doencaCardiaca);
                    binding.push(u.cardiacaInsuficiencia);
                    binding.push(u.cardiacaOutros);
                    binding.push(u.cardiacaNaoSabe);
                    binding.push(u.doencaRins);
                    binding.push(u.rinsInsuficiencia);
                    binding.push(u.rinsOutros);
                    binding.push(u.rinsNaoSabe);
                    binding.push(u.doencaRespiratoria);
                    binding.push(u.respiratoriaAsma);
                    binding.push(u.respiratoriaEfisema);
                    binding.push(u.respiratoriaOutros);
                    binding.push(u.respiratoriaNaoSabe);
                    binding.push(u.outrasPraticasIntegrativas);
                    binding.push(u.condicaoSaude1);
                    binding.push(u.condicaoSaude2);
                    binding.push(u.condicaoSaude3);
                    binding.push(u.possuiDeficiencia);
                    binding.push(u.informaOrientacaoSexual);
                    binding.push(u.possuiSofrimentoPsiquicoGrave);
                    binding.push(u.utilizaProtese);
                    binding.push(u.proteseAuditiva);
                    binding.push(u.proteseMembrosSuperiores);
                    binding.push(u.proteseMembrosInferiores);
                    binding.push(u.proteseCadeiraRodas);
                    binding.push(u.proteseOutros);
                    binding.push(u.parentescoResponsavel);

                    binding.push(u.paiDesconhecido);
                    binding.push(u.maeDesconhecida);
                    binding.push(u.dataNaturalizado);
                    binding.push(u.portariaNaturalizacao);
                    binding.push(u.dtEntradaBrasil);
                    binding.push(u.dataObito);
                    binding.push(u.numeroDO);
                    binding.push(u.identidadeGenero);

                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("PacienteEsus", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.pacienteDado = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "dataDados": null,
                "peso": parseInt(registro[3]),
                "altura": parseInt(registro[4])
            };
            if (registro[2] !== '') {
                var date = registro[2].substring(0, 10).split('-');
                var hour = registro[2].substring(11).split(':');
                itens[i].dataDados = new Date(date[2], date[1] - 1, date[0], hour[0], hour[1], hour[2]).getTime() / 1000;
            }
        }
        linha = null;

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO pacienteDado " +
                        "(id, versao, dataColeta, peso, altura, codigoPaciente)" +
                        " VALUES (?, ?, ?, ?, ?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?))";
                    binding.push(u.id);
                    binding.push(u.versao);
                    binding.push(u.dataDados);
                    binding.push(u.peso);
                    binding.push(u.altura);
                    binding.push(u.codigoSistema);
                } else {
                    sql = "INSERT OR REPLACE INTO pacienteDado " +
                        "(id, versao, dataColeta, peso, altura, codigoPaciente)" +
                        "SELECT CASE WHEN p.codigoSistema = ? THEN pd.id ELSE null END ," +
                        "?,?,?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)" +
                        " FROM pacienteDado pd LEFT JOIN paciente p ON pd.codigoPaciente = p.id WHERE p.codigoSistema = ? OR NOT EXISTS(SELECT pd2.id FROM pacienteDado pd2 LEFT JOIN paciente p2 WHERE p2.codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.dataDados);
                    binding.push(u.peso);
                    binding.push(u.altura);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Paciente Dado", errorMessage, e);
            deferred.reject(e);
        }, function() {
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.notificacaoPaciente = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Notificação(ões) Paciente');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": registro[0],
                "versao": parseInt(registro[1]),
                "codigoPaciente": parseInt(registro[2]),
                "vacinaAtrasada": parseInt(registro[3])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO notificacaoPaciente " +
                        "(id, codigoSistema, versao, codigoPaciente, vacinaAtrasada) " +
                        "VALUES ((SELECT id FROM paciente WHERE codigoSistema = ?), ?, ?," +
                        "?, ?)";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoPaciente);
                    binding.push(u.vacinaAtrasada);

                } else {
                    sql = "INSERT OR REPLACE INTO notificacaoPaciente " +
                        "(id, codigoSistema, versao, codigoPaciente, vacinaAtrasada) " +
                        "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                        u.codigoSistema + "," + u.versao + "," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)," +
                        +u.vacinaAtrasada + " FROM notificacaoPaciente WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM notificacaoPaciente WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoPaciente);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Notificações Paciente", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Notificações Paciente importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.notificacaoAgendas = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Notificação(ões) Agenda');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": registro[0],
                "versao": parseInt(registro[1]),
                "codigoAgenda": parseInt(registro[2]),
                "codigoPaciente": parseInt(registro[3]),
                "status": parseInt(registro[4]),
                "dataAgendamento": null,
                "local": emptyToNull(registro[6]),
                "tipoAgenda": emptyToNull(registro[7])
            };
            if (registro[5] !== '') {
                var date = registro[5].substring(0, 10).split('-');
                var hora = registro[5].substring(10).split(':');
                itens[i].dataAgendamento = new Date(date[2], date[1] - 1, date[0], hora[0], hora[1], hora[2], 0).getTime() / 1000;
            }
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO notificacaoAgendas " +
                        "(id, codigoSistema, versao, codigoAgenda, codigoPaciente, status, tipoAgenda, local, dataAgendamento) " +
                        "VALUES (?, ?, ?,?," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?), ?,?,?,?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoAgenda);
                    binding.push(u.codigoPaciente);
                    binding.push(u.status);
                    binding.push(u.tipoAgenda);
                    binding.push(u.local);
                    binding.push(u.dataAgendamento);

                } else {
                    sql = "INSERT OR REPLACE INTO notificacaoAgendas " +
                        "(id, codigoSistema, versao, codigoAgenda, codigoPaciente, status, tipoAgenda, local, dataAgendamento) " +
                        "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                        u.codigoSistema + "," + u.versao + "," + u.codigoAgenda + "," +
                        "(SELECT id FROM paciente WHERE codigoSistema = ?)," +
                        +u.status + ",\"" + u.tipoAgenda + "\",\"" + u.local + "\", ? FROM notificacaoAgendas WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM notificacaoAgendas WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoPaciente);
                    binding.push(u.dataAgendamento);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Notificações Agenda", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Notificações Agenda importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.motivoNaoComparecimento = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Motivo(s) Não Comparecimento', 'motivoNaoComparecimento', 'Motivos de Não Comparecimento', 'Motivos de Não Comparecimento', [
            {name: "outros", type: "emptyToNull"},
            {name: "descricao", type: "emptyToNull"},
        ]);
    }

    parsersService.tipoVacina = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Tipo(s) Vacina', 'tipoVacina', 'Tipo de Vacina', 'Tipos de Vacina', [
            {name: "descricao", type: "emptyToNull"},
        ]);
    }

    parsersService.pesquisa = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Pesquisa(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "descricao": emptyToNull(registro[2]),
                "inicio": null,
                "fim": null,
                "status": parseInt(registro[5])
            };
            if (registro[3] !== '') {
                var date = registro[3].split('-');
                itens[i].inicio = new Date(date[2], date[1] - 1, date[0], null, null, null, 0).getTime() / 1000;
            }
            if (registro[4] !== '') {
                var date = registro[4].split('-');
                itens[i].fim = new Date(date[2], date[1] - 1, date[0], null, null, null, 0).getTime() / 1000;
            }
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO pesquisa " +
                        "(id, codigoSistema, versao, descricao, inicio, fim, status) " +
                        "VALUES (?, ?, ?, ?, ?, ?, ?)";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.descricao);
                    binding.push(u.inicio);
                    binding.push(u.fim);
                    binding.push(u.status);

                } else {
                    sql = "INSERT OR REPLACE INTO pesquisa " +
                        "(id, codigoSistema, versao, descricao, inicio, fim, status) " +
                        "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                        u.codigoSistema + "," + u.versao + ",\"" + u.descricao + "\"" + "," + u.inicio + "," + u.fim + "," + u.status +
                        " FROM pesquisa WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM pesquisa WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        $dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Pesquisas", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Pesquisas importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    }

    parsersService.perguntaPesquisa = function(data, novo) {
        return parsersService.defaultParser(data, novo, 'Pergunta(s)', 'perguntaPesquisa', 'Perguntas', 'Perguntas', [
            {name: "descricao", type: "emptyToNull"},
        ]);
    };

    parsersService.perguntaResposta = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Resposta(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "descricao": emptyToNull(registro[2]),
                "codigoPergunta": parseInt(registro[3])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO perguntaResposta " +
                        "(id, codigoSistema, versao, descricao, codigoPergunta) " +
                        "VALUES (?, ?, ?, ?, " +
                        " (SELECT id FROM perguntaPesquisa WHERE codigoSistema = ?))";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.descricao);
                    binding.push(u.codigoPergunta);

                } else {
                    sql = "INSERT OR REPLACE INTO perguntaResposta " +
                        "(id, codigoSistema, versao, descricao, codigoPergunta) " +
                        "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                        u.codigoSistema + "," + u.versao + ",\"" + u.descricao + "\", (SELECT id FROM perguntaPesquisa WHERE codigoSistema = ?)" +
                        " FROM perguntaResposta WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM perguntaResposta WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoPergunta);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Respostas", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Respostas importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    };

    parsersService.pesquisaPergunta = function(data, novo) {
        var deferred = $q.defer();

        var itens = [];
        var linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Pergunta(s) das pesquisa');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "codigoPesquisa": parseInt(registro[2]),
                "codigoPergunta": parseInt(registro[3])
            };
        }

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var sql;
                if (novo) {
                    sql = "INSERT OR REPLACE INTO pesquisaPergunta " +
                        "(id, codigoSistema, versao, codigoPesquisa, codigoPergunta) " +
                        "VALUES (?, ?, ?, " +
                        " (SELECT id FROM pesquisa WHERE codigoSistema = ?), " +
                        " (SELECT id FROM perguntaPesquisa WHERE codigoSistema = ?))";
                    binding.push(u.id);
                    binding.push(u.codigoSistema);
                    binding.push(u.versao);
                    binding.push(u.codigoPesquisa);
                    binding.push(u.codigoPergunta);

                } else {
                    sql = "INSERT OR REPLACE INTO pesquisaPergunta " +
                        "(id, codigoSistema, versao, codigoPesquisa, codigoPergunta) " +
                        "SELECT CASE WHEN codigoSistema = " + u.codigoSistema + " THEN id ELSE null END ," +
                        u.codigoSistema + "," + u.versao + ", (SELECT id FROM pesquisa WHERE codigoSistema = ?), (SELECT id FROM perguntaPesquisa WHERE codigoSistema = ?)" +
                        " FROM pesquisaPergunta WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM pesquisaPergunta WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoPesquisa);
                    binding.push(u.codigoPergunta);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("Perguntas da pesquisa", errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift('Perguntas das pesquisa importadas com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    };

    parsersService.exclusoes = function(data, novo) {
        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' exclusão(ões)');
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
                "tipoRegistro": emptyToNull(registro[2]).replace('_', ''),
                "idRegistro": emptyToNull(registro[3])
            };
        }

        for (var i = 0; i < itens.length; i++) {
            var u = itens[i];
            var binding = [];
            var sql = "DELETE FROM " + u.tipoRegistro +
                " WHERE codigoSistema = ? ";
            binding.push(u.idRegistro);

            DB.query(sql, binding, null, function(err) {
                dataService.data.erros.push('Erro ao excluir ' + u.tipoRegistro);
                dataService.data.erros.push(err.message);
            });
        }

        if (itens.length > 0) {
            storageService.setItem("versaoExclusao", itens[itens.length - 1].versao);
        }
    };

    parsersService.dominioPaciente = function(data, novo, totalCount, totalLength) {
        var deferred = $q.defer();

        var itens = [];
        linha = data;
        // dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' Domínio Paciente(s)');
        for (var i = 0; i < linha.length - 1; i++) {
            var registro = linha[i].split('|');
            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "nome": emptyToNull(registro[1]),
                "dataNascimento": emptyToNull(registro[2]),
                "nomeMae": emptyToNull(registro[3]),
                "cns": emptyToNull(registro[4]),
                "cpf": emptyToNull(registro[5]),
                "descricaoFamilia": emptyToNull(registro[6]),
                "sexo": emptyToNull(registro[9]),
                "ativo": emptyToNull(registro[10]),
                "keyword": registro[1] + ' ' + registro[2] + ' ' + registro[3] + ' ' + registro[4] + ' ' + registro[5],
                "versao": parseInt(registro[8]),
                "codigoMicroArea": parseInt(registro[11]),
                "codigoArea": parseInt(registro[12])
            };

            if (registro[2] && registro[2] !== '') {
                var date = registro[2].split('-');
                itens[i].dataNascimento = new Date(date[2], date[1] - 1, date[0]).getTime() / 1000;
            }
        }

        var errorMessage = '';

        // binding.push();
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = totalCount || 0;
                var binding = [];
                var sql;

                if (novo) {
                    sql = "INSERT OR REPLACE INTO dominioPaciente " +
                        "(id, codigoSistema, nome, dataNascimento, nomeMae, cns, cpf, descricaoFamilia, sexo, ativo, keyword, versao, codigoMicroArea, codigoArea, codigoInterno) " +
                        "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?,?, ?, ?, ?, ?, (SELECT id FROM paciente WHERE codigoSistema = ?))";
                    for (var key in u) {
                        if (u.hasOwnProperty(key)) {
                            binding.push(u[key]);
                        }
                    }

                    binding.push(u['codigoSistema']);

                } else {
                    sql = "INSERT OR REPLACE INTO dominioPaciente " +
                        "(id, codigoSistema, nome, dataNascimento, nomeMae, cns, cpf, descricaoFamilia, sexo, ativo, keyword, versao, codigoMicroArea, codigoArea, codigoInterno) " +
                        "SELECT CASE WHEN codigoSistema = ? THEN id ELSE null END ," +
                        "?,?,?, ?, ?, ?, ?, ?, ?, ?, ?,?, ?, (SELECT id FROM paciente WHERE codigoSistema = ?)" +
                        "FROM dominioPaciente WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM dominioPaciente WHERE codigoSistema = ?) LIMIT 1";
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.nome);
                    binding.push(u.dataNascimento);
                    binding.push(u.nomeMae);
                    binding.push(u.cns);
                    binding.push(u.cpf);
                    binding.push(u.descricaoFamilia);
                    binding.push(u.sexo);
                    binding.push(u.ativo);
                    binding.push(u.keyword);
                    binding.push(u.versao);
                    binding.push(u.codigoMicroArea);
                    binding.push(u.codigoArea);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 1000 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + totalLength + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro("dominio paciente", errorMessage, e);
            deferred.reject(e);
        }, function() {
            //dataService.data.log.unshift('DominioPaciente importados com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    };

    parsersService.setPropertiesPaciente = function(paciente) {
        var binding = [];

        binding.push(paciente.versao);
        binding.push(paciente.excluido);
        binding.push(paciente.nome);
        binding.push(paciente.dataNascimento);
        binding.push(paciente.sexo);
        binding.push(paciente.nomeMae);
        binding.push(paciente.codigoCidade);
        binding.push(paciente.cpf);
        binding.push(paciente.rg);
        binding.push(paciente.telefone);
        binding.push(paciente.celular);
        binding.push(paciente.codigoRaca);
        binding.push(paciente.codigoPais);
        binding.push(paciente.email);
        binding.push(paciente.apelido);
        binding.push(paciente.flagResponsavelFamiliar);
        binding.push(paciente.nacionalidade);
        binding.push(paciente.codigoDomicilio);
        binding.push(paciente.telefone2);
        binding.push(paciente.telefone3);
        binding.push(paciente.telefone4);
        binding.push(paciente.religiao);
        binding.push(paciente.localTrabalho);
        binding.push(paciente.telefoneTrabalho);
        binding.push(paciente.responsavel);
        binding.push(paciente.parentescoResponsavel);
        binding.push(paciente.urgenciaNome);
        binding.push(paciente.urgenciaTelefone);
        binding.push(paciente.urgenciaParentesco);
        binding.push(paciente.rendaFamiliar);
        binding.push(paciente.resideDesde);
        binding.push(paciente.situacao);
        binding.push(paciente.nis);
        binding.push(paciente.prontuario);
        binding.push(paciente.motivoExclusao);
        binding.push(paciente.codigoEtniaIndigena);
        binding.push(paciente.keyword.latinise());
        binding.push(paciente.codigoResponsavelFamiliar);
        binding.push(paciente.nomePai);

        return binding;
    };

    parsersService.updatePaciente = function(tx, pac) {
        var binding = [];
        var sql = "UPDATE paciente  SET " +
            " versao = ?, excluido = ?, nome = ?, dataNascimento = ?, sexo = ?, nomeMae = ?, " +
            " codigoCidade = (SELECT id FROM cidade WHERE codigoSistema = ?)," +
            " cpf = ?, rg = ?, telefone = ?, celular = ?, " +
            " codigoRaca = (SELECT id FROM raca WHERE codigoSistema = ?)," +
            " codigoPais = (SELECT id FROM pais WHERE codigoSistema = ?)," +
            " email = ?, apelido = ?, flagResponsavelFamiliar = ?, nacionalidade = ?, " +
            " codigoDomicilio = (SELECT id FROM domicilio WHERE codigoSistema = ?), " +
            " telefone2 = ?," +
            " telefone3 = ?, telefone4 = ?, religiao = ?, localTrabalho = ?, telefoneTrabalho = ?, responsavel = ?, parentescoResponsavel = ?, " +
            " urgenciaNome = ?, urgenciaTelefone = ?, urgenciaParentesco = ?, rendaFamiliar = ?, resideDesde = ?, situacao = ?, nis = ?, prontuario = ?, motivoExclusao = ?," +
            " codigoEtniaIndigena = (SELECT id FROM etniaIndigena WHERE codigoSistema = ?)," +
            " keyword = ?, codigoResponsavelFamiliar = (SELECT id FROM paciente WHERE codigoSistema = ?), nomePai =? " +
            " WHERE codigoSistema = ?;";

        binding.extend(parsersService.setPropertiesPaciente(pac));
        binding.push(pac.codigoSistema);

        tx.executeSql(sql, binding, function(transaction, result) {
            if (result.rowsAffected <= 0) {
                var sqlInsert = "INSERT OR REPLACE INTO paciente " +
                    "(codigoSistema, versao, excluido, nome, dataNascimento, sexo, nomeMae, codigoCidade," +
                    "cpf, rg, telefone, celular, codigoRaca, codigoPais, email, apelido," +
                    "flagResponsavelFamiliar, nacionalidade, codigoDomicilio, telefone2," +
                    "telefone3, telefone4, religiao, localTrabalho, telefoneTrabalho, responsavel, parentescoResponsavel," +
                    "urgenciaNome, urgenciaTelefone, urgenciaParentesco, rendaFamiliar, resideDesde, situacao, nis, prontuario, motivoExclusao, codigoEtniaIndigena, keyword, codigoResponsavelFamiliar, nomePai)" +
                    "SELECT ?,?,?,?,?,?,?," +
                    "(SELECT id FROM cidade WHERE codigoSistema = ?)," +
                    "?,?,?,?," +
                    "(SELECT id FROM raca WHERE codigoSistema = ?)," +
                    "(SELECT id FROM pais WHERE codigoSistema = ?)," +
                    "?,?,?,?," +
                    "(SELECT id FROM domicilio WHERE codigoSistema = ?)," +
                    "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?," +
                    "(SELECT id FROM etniaIndigena WHERE codigoSistema = ?)," +
                    "?, (SELECT id FROM paciente WHERE codigoSistema = ?), ?" +
                    " FROM paciente LIMIT 1";

                var bindingInsert = [];
                bindingInsert.push(pac.codigoSistema);
                bindingInsert.extend(parsersService.setPropertiesPaciente(pac));

                tx.executeSql(sqlInsert, bindingInsert, function(transaction, result) {}, function(transaction, error) {
                    dataService.data.log.unshift(error.message);
                    return true;
                });
            }
        }, function(transaction, error) {
            dataService.data.log.unshift(error.message);
            return true;
        });
    };

    parsersService.defaultParser = function(data, novo, itemLog, itemTable, itemError, itemSuccess, itemColumns) {
        var deferred = $q.defer();

        var itens = [];
        linha = data.split('\n');
        dataService.data.log.unshift('Importando ' + (linha.length - 1) + ' ' + itemLog);
        for (var i = 0; i < linha.length - 1; i++) {
            registro = linha[i].split('|');

            itens[i] = {
                "id": null,
                "codigoSistema": parseInt(registro[0]),
                "versao": parseInt(registro[1]),
            };

            angular.forEach(itemColumns, function(column, key) {
                if (column.type === 'parseInt')
                    itens[i][column.name] = parseInt(registro[key+2]);
                else if (column.type === 'emptyToNull')
                    itens[i][column.name] = emptyToNull(registro[key+2]);
                else if (column.type === 'value')
                    itens[i][column.name] = column.value;
                else
                    itens[i][column.name] = registro[key+2];
            });
        };

        var errorMessage = '';
        DB.session.transaction(function(tx) {
            for (var i = 0; i < itens.length; i++) {
                var u = itens[i];
                var count = 0;
                var binding = [];
                var itensLength = Object.keys(u).length;
                var sql = "INSERT OR REPLACE INTO "+ itemTable +" (id, codigoSistema, versao";
                var index = 0;
                if (novo) {
                    var sqlValues = "VALUES (?, ?, ?";

                    for (var key in u) {
                        if (index > 2) {
                            sql += " ," + key;
                            sqlValues += ", ?";
                        }

                        index++;

                        if (u.hasOwnProperty(key)) {
                            binding.push(u[key]);
                        }
                    }

                    sql += ") " + sqlValues + ") ";

                } else {
                    var sqlValues = "SELECT CASE WHEN codigoSistema = \"" + u.codigoSistema + "\" THEN id ELSE null END ,\"" +
                    u.codigoSistema + "\"," + u.versao;

                    for (var key in u) {

                        if (index > 2) {
                            sql += " ," + key;

                        if (Number.isInteger(u[key]))
                            sqlValues += ","+u[key];
                        else
                            sqlValues += ",\""+u[key]+"\"";
                        }

                        index++;
                    }

                    sql += ") " + sqlValues + " FROM "+ itemTable +" WHERE codigoSistema = ? OR NOT EXISTS(SELECT id FROM "+ itemTable +" WHERE codigoSistema = ?) LIMIT 1";

                    binding.push(u.codigoSistema);
                    binding.push(u.codigoSistema);
                }

                tx.executeSql(sql, binding, function(transaction, result) {
                    count++;
                    if (count % 100 === 0 && errorMessage === '') {
                        dataService.data.log.unshift(count + ' de ' + itens.length + ' importados');
                    }
                }, function(transaction, error) {
                    if (errorMessage === '') {
                        errorMessage = error.message;
                    }
                    return true;
                });
            }
        }, function(e) {
            _mostrarErro(itemError, errorMessage, e);
            deferred.reject(e);
        }, function() {
            dataService.data.log.unshift(itemSuccess +' importados com sucesso!');
            deferred.resolve();
        });

        return deferred.promise;
    };

    return parsersService;
});