services.factory('updateService', function (storageService, messageService, $ionicLoading, $http, $state, $cordovaFile, $cordovaFileOpener2) {
    var updateService = this;

    //Adquire o dominio da url
    var _getDominio = function () {
        if (!storageService.getItem("dominio")) {
            //URL salva na tela inicial de sicronização
            var host = storageService.getItem("url");
            //Verifica se há um dominio
            var domain = function () {
                //Procura por uma palavra, multiplas vezes pegando apenas a primeira ocorrencia ignorando maiuscula ou minuscula, onde a palavra terminar com ponto
                if (!host.match(/[\w]+\./i)) {
                    return null;
                }
                // Atribui o regex, remove o ponto e deixa o dominio em minusculo
                return host.match(/[\w]+\./i)[0].split('.').join("").toLowerCase();

            };
            //Pega o ip da url
            var ipMatch = function () {
                //Pega no formato ip a localização, caso for ip retorna localhost
                if (/(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)/i.test(host)) {
                    return 'localhost';
                }
            };
            //Tenta achar o ip, se não encontrar procura o dominio
            storageService.setItem("dominio", !!ipMatch() ? ipMatch() : domain());
        }
        return storageService.getItem("dominio").replace("teste", "").replace("treinamento", "");

    };

    var _getUrlPrincipal = function () {
        var urlPrincipal = 'https://s3-sa-east-1.amazonaws.com/celk/storage/aplicativos/familias2/';
        return urlPrincipal + _getDominio() + "/";
    };

    updateService.verificarAtualizacao = function ($scope) {
        $http.get('versao.json').success(function (data) {
            $scope.bind.versaoLocal = data.versaoAtual;
            var urlPrincipal = _getUrlPrincipal();
            var versaoLocal = data.versaoAtual;
            var url = urlPrincipal + 'versao.json?' + new Date().getTime();
            var urlApk = urlPrincipal + 'familias.apk';
            storageService.setItem('versaoLocal', data.versaoAtual);
            console.log('VERSAO LOCAL: ' + $scope.bind.versaoLocal);
            console.log('URL REMOTA: ' + url);

            var _downloadApk = function () {
                $scope.bind.loading = 'Carregando...';
                $ionicLoading.show({
                    template: '{{bind.loading}}',
                    scope: $scope
                });
                $cordovaFile.downloadFile(urlApk,
                    cordova.file.externalDataDirectory + '/familias.apk').then(function (result) {

                    $cordovaFileOpener2.open(
                        cordova.file.externalDataDirectory + '/familias.apk',
                        'application/vnd.android.package-archive'
                    );

                    $ionicLoading.hide();

                }, function (err) {
                    $ionicLoading.hide();
                    messageService.mostrarAlerta('Não foi possível atualizar, verifique se você está conectado na internet e tente novamente.');
                }, function (progress) {
                    if (progress.lengthComputable) {
                        var perc = Math.floor(progress.loaded / progress.total * 100);
                        $scope.bind.loading = 'Carregando... ' + perc + '%';
                    } else {
                        $scope.bind.loading = 'Carregando... ';
                    }
                });
            };

            $http.get(url).success(function (data) {
                var versaoRemota = data.versaoAtual;
                var versaoMinimaRemota = data.versaoMinima;
                console.log('VERSAO REMOTA: ' + versaoRemota);
                console.log('VERSAO MINIMA: ' + versaoMinimaRemota);
                if (parseInt(versaoMinimaRemota) > parseInt(versaoLocal) && $state.current.name == 'menu') {
                    storageService.setItem('desabilitaSincronizacao', true);
                    $scope.desabilitaSincronizacao = true;
                    messageService.mostrarConfirmacao('Para continuar usando o aplicativo é necessário atualizar. Deseja atualizar agora?').then(function (res) {
                        if (res) {
                            _downloadApk();
                        }
                    });
                } else if (!isNaN(versaoRemota) && !isNaN(versaoLocal) && parseInt(versaoRemota) > parseInt(versaoLocal)) {
                    if ($state.current.name == 'menu') {
                        messageService.mostrarConfirmacao('Existe uma nova versão disponível que pode corrigir problemas no aplicativo. Deseja atualizar agora?').then(function (res) {
                            if (res) {
                                _downloadApk();
                            }
                        });
                    }
                } else {
                    storageService.removeItem('desabilitaSincronizacao');
                    delete $scope.desabilitaSincronizacao;
                }

            }).error(function (data, status) {
                if ($scope.desabilitaSincronizacao && status !== 200) {
                    storageService.removeItem('desabilitaSincronizacao');
                    delete $scope.desabilitaSincronizacao;
                } else if (status === 403) {
                    messageService.mostrarAlerta('Cliente (' + _getDominio() + ') não configurado para atualização, por favor entre em contato com o suporte.');
                }
            });
        });
    };

    return updateService;
});