/**
 * Implementações de regras do esus baseados em http://go-h.ws/esus-helper e http://go-h.ws/regras-esus
 */
services.factory('validacaoService', function (storageService) {
    var self = this;

    /**
     * Valida se o nome social é válido com base na Regra 6 do e-sus
     * @param {string} nomeSocial O nome que será validado
     */
    self.isNomeSocialValido = function (nomeSocial) {
        //#region Regra 6
        /*  6. A informação só pode conter letras do alfabeto romano (incluindo K, W, Y),
         *      além dos acentos gráficos (agudo, circunflexo, til e trema) e do caractere apóstrofo (’)
         *      para possibilitar a inserção de nomes como JOÃO D’ÁVILA, sendo vedada a utilização de outros caracteres especiais; */
        var caracteresPermitidos = /[^\sa-záàãâäéèẽêëíìĩîïóòõôöúùũûüçÁÀÃÂÄÉÈẼÊËÍÌĨÎÏÓÒÕÖÔÚÙŨÛÜÇ’']/gi
        if (caracteresPermitidos.test(nomeSocial)) {
            return false;
        }
        nomeSocial = nomeSocial.normalize('NFD').replace(/[\u0300-\u036f]/gi, "").toLowerCase();
        //Remove os acentos e Transforma em minusculos
        if (nomeSocial.trim() == "" || !nomeSocial) {
            return false;
        }
        return nomeSocial;
        //#endregion

    };
    /**
     * Retorna se o nome é valido
     * @param {string} nome O nome que será validado
     */
    self.isNomeValido = function (nome) {

        nome = nome.trim();
        if (nome == "" || !nome) {
            return false;
        }
        nome = self.isNomeSocialValido(nome);
        if(!nome){
            return false;
        }
        console.log(nome)
        //#region Regra 4
        var names = nome.split(/\s/);

        //  4. A informação que contiver um único termo, não entrará na base de dados do CADSUS (Ex: JOAQUIM);
        if (names.length < 2) {
            return false;
        }

        var firstName = names[0];
        if (firstName.length == 1) {
            return false;

        }
        //#endregion

        //#region Regra 3 Espaço duplo
        if (/\s\s+/g.test(nome)) {
            return false;
        }
        //#endregion

        //#region Regra 14
        if (names.length == 2) {
            //  14. A informação que contiver apenas dois termos, ambos com apenas dois caracteres, não entrará na base de dados do CADSUS;
            if (names[1].length == 1 || (firstName.length == 2 && names[1].length == 2)) {
                return false;
            }
        }

        var lastName = names[names.length - 1];
        if (lastName.length == 1) {
            return false;
        }
        //#endregion

        //#region Regra 15 e 16
        var regraConsoanteVogal = /^[^aeyiuo]+$|[^aeyiuo]{4,}[aeyiuo]/gi
        if (regraConsoanteVogal.test(firstName) && firstName !== "rn" && firstName.indexOf("w") === -1 && firstName.indexOf("y") === -1) {
            /*  15. A informação que contiver o 1° termo com apenas consoantes, não entrará na base de dados do CADSUS (Ex: PFTG SANTOS MARTINS),
             *       exceto nos casos de: 1 - esse 1º termo conter as letras W e Y; e 2 – esse termo for igual a “RN”;
             *   16. A informação que contiver o 1° termo com 4 ou mais consoantes seguidas de vogal não entrará na base de dados do CADSUS (Ex: PFTGAHQ MOREIRA),
             *       exceto no caso dessa informação conter as letras: W e Y. */

            return false;
        }
        //#endregion

        //#region Regra 5
        /*  5. A informação que contiver, após o 1º termo, um termo com apenas um caractere não entrará na base de dados do CADSUS,
               exceto se o termo for a letra E ou Y (Ex: JOAQUIM MARIA A SILVA não entra, mas JOAQUIM MARIA E SILVA é admissível; JOAQUIM RODRIGUEZ Y RODRIGUEZ é admissível); */

        var BreakException = {};
        try {
            names.forEach(function (element) {
                if (element.length == 1) {
                    if (element.indexOf("e") !== -1 || element.indexOf("y") !== -1) {
                        return;
                    }
                    throw BreakException;
                }
            });
        } catch (e) {
            if (e === BreakException) {
                return false;
            } else {
                throw e;
            }
        }

        return true;
    };
    //#endregion

    /**
     * Limpa o telefone caso for invalido
     * @param {string} telefone 
     */
    self.limparTelefone = function (telefone) {
        if (!telefone) {
            return telefone;
        }

        if (telefone.length < 10 || telefone.length > 11) {
            telefone = undefined;
        }

        return telefone

    };

    self.isEmailValido = function(email){
        return /^[\w-]+(\.[\w-]+)*@([\w-]+\.)+[a-zA-Z]{2,7}$/g.test(email);
    };

    self.isDev = function () {
        if(storageService.getItem('dominio')){
            return storageService.getItem('dominio').indexOf('teste') > -1 || storageService.getItem('dominio').indexOf('treinamento') > -1 || storageService.getItem('dominio').indexOf('localhost') > -1;
        }
    };

    self.isProtuarioValido = function (prontuario) {
        return !/[^a-z0-9]/gi.test(prontuario);
    }

    return self;
});